@@ -18,20 +18,25 @@
 package org.apache.zeppelin.spark
 
 
-import java.io.File
-import java.net.URLClassLoader
+import java.io.{File, IOException}
+import java.net.{URL, URLClassLoader}
 import java.nio.file.Paths
 import java.util.concurrent.atomic.AtomicInteger
 
+import org.apache.commons.lang3.StringUtils
+import org.apache.hadoop.yarn.client.api.YarnClient
+import org.apache.hadoop.yarn.conf.YarnConfiguration
+import org.apache.hadoop.yarn.util.ConverterUtils
+import org.apache.hadoop.conf.Configuration
+import org.apache.hadoop.fs.{FileSystem, Path}
 import org.apache.spark.sql.SQLContext
-import org.apache.spark.{JobProgressUtil, SparkConf, SparkContext}
-import org.apache.zeppelin.interpreter.thrift.InterpreterCompletion
+import org.apache.spark.{SparkConf, SparkContext}
 import org.apache.zeppelin.interpreter.util.InterpreterOutputStream
-import org.apache.zeppelin.interpreter.{InterpreterContext, InterpreterResult}
+import org.apache.zeppelin.interpreter.{InterpreterContext, InterpreterGroup, InterpreterResult, ZeppelinContext}
 import org.slf4j.{Logger, LoggerFactory}
 
 import scala.collection.JavaConverters._
-import scala.tools.nsc.interpreter.Completion.ScalaCompleter
+import scala.tools.nsc.interpreter.Completion
 import scala.util.control.NonFatal
 
 /**
@@ -40,29 +45,40 @@ import scala.util.control.NonFatal
   *
   * @param conf
   * @param depFiles
+  * @param properties
+  * @param interpreterGroup
   */
 abstract class BaseSparkScalaInterpreter(val conf: SparkConf,
                                          val depFiles: java.util.List[String],
-                                         val printReplOutput: java.lang.Boolean) {
+                                         val properties: java.util.Properties,
+                                         val interpreterGroup: InterpreterGroup,
+                                         val sparkInterpreterClassLoader: URLClassLoader)
+  extends AbstractSparkScalaInterpreter() {
 
   protected lazy val LOGGER: Logger = LoggerFactory.getLogger(getClass)
 
-  private val isTest = conf.getBoolean("zeppelin.spark.test", false)
-
   protected var sc: SparkContext = _
 
   protected var sqlContext: SQLContext = _
 
   protected var sparkSession: Object = _
 
+  protected var outputDir: File = _
+
+  protected var userJars: Seq[String] = _
+
   protected var sparkHttpServer: Object = _
 
   protected var sparkUrl: String = _
 
-  protected var scalaCompleter: ScalaCompleter = _
+  protected var scalaCompletion: Completion = _
+
+  protected var z: SparkZeppelinContext = _
 
   protected val interpreterOutput: InterpreterOutputStream
 
+  protected val sparkMaster: String = conf.get(SparkStringConstants.MASTER_PROP_NAME,
+    SparkStringConstants.DEFAULT_MASTER_VALUE)
 
   protected def open(): Unit = {
     /* Required for scoped mode.
@@ -89,13 +105,17 @@ abstract class BaseSparkScalaInterpreter(val conf: SparkConf,
   def interpret(code: String, context: InterpreterContext): InterpreterResult = {
 
     val originalOut = System.out
+    val printREPLOutput = context.getStringLocalProperty("printREPLOutput", "true").toBoolean
 
     def _interpret(code: String): scala.tools.nsc.interpreter.Results.Result = {
       Console.withOut(interpreterOutput) {
         System.setOut(Console.out)
-        interpreterOutput.setInterpreterOutput(context.out)
+        if (printREPLOutput) {
+          interpreterOutput.setInterpreterOutput(context.out)
+        } else {
+          interpreterOutput.setInterpreterOutput(null)
+        }
         interpreterOutput.ignoreLeadingNewLinesFromScalaReporter()
-        context.out.clear()
 
         val status = scalaInterpret(code) match {
           case success@scala.tools.nsc.interpreter.IR.Success =>
@@ -122,6 +142,7 @@ abstract class BaseSparkScalaInterpreter(val conf: SparkConf,
     // reset the java stdout
     System.setOut(originalOut)
 
+    context.out.write("")
     val lastStatus = _interpret(code) match {
       case scala.tools.nsc.interpreter.IR.Success =>
         InterpreterResult.Code.SUCCESS
@@ -131,7 +152,10 @@ abstract class BaseSparkScalaInterpreter(val conf: SparkConf,
         InterpreterResult.Code.INCOMPLETE
     }
 
-    new InterpreterResult(lastStatus)
+    lastStatus match {
+      case InterpreterResult.Code.INCOMPLETE => new InterpreterResult( lastStatus, "Incomplete expression" )
+      case _ => new InterpreterResult(lastStatus)
+    }
   }
 
   protected def interpret(code: String): InterpreterResult =
@@ -139,18 +163,20 @@ abstract class BaseSparkScalaInterpreter(val conf: SparkConf,
 
   protected def scalaInterpret(code: String): scala.tools.nsc.interpreter.IR.Result
 
-  protected def completion(buf: String,
-                           cursor: Int,
-                           context: InterpreterContext): java.util.List[InterpreterCompletion] = {
-    val completions = scalaCompleter.complete(buf.substring(0, cursor), cursor).candidates
-      .map(e => new InterpreterCompletion(e, e, null))
-    scala.collection.JavaConversions.seqAsJavaList(completions)
-  }
-
   protected def getProgress(jobGroup: String, context: InterpreterContext): Int = {
     JobProgressUtil.progress(sc, jobGroup)
   }
 
+  override def getSparkContext: SparkContext = sc
+
+  override def getSqlContext: SQLContext = sqlContext
+
+  override def getSparkSession: AnyRef = sparkSession
+
+  override def getSparkUrl: String = sparkUrl
+
+  override def getZeppelinContext: ZeppelinContext = z
+
   protected def bind(name: String, tpe: String, value: Object, modifier: List[String]): Unit
 
   // for use in java side
@@ -161,19 +187,41 @@ abstract class BaseSparkScalaInterpreter(val conf: SparkConf,
     bind(name, tpe, value, modifier.asScala.toList)
 
   protected def close(): Unit = {
-    if (BaseSparkScalaInterpreter.sessionNum.decrementAndGet() == 0) {
-      if (sc != null) {
-        sc.stop()
+    // delete stagingDir for yarn mode
+    if (sparkMaster.startsWith("yarn")) {
+      val hadoopConf = new YarnConfiguration()
+      val appStagingBaseDir = if (conf.contains("spark.yarn.stagingDir")) {
+        new Path(conf.get("spark.yarn.stagingDir"))
+      } else {
+        FileSystem.get(hadoopConf).getHomeDirectory()
       }
-      if (sparkHttpServer != null) {
-        sparkHttpServer.getClass.getMethod("stop").invoke(sparkHttpServer)
-      }
-      sc = null
-      sqlContext = null
-      if (sparkSession != null) {
-        sparkSession.getClass.getMethod("stop").invoke(sparkSession)
-        sparkSession = null
+      val stagingDirPath = new Path(appStagingBaseDir, ".sparkStaging" + "/" + sc.applicationId)
+      cleanupStagingDirInternal(stagingDirPath, hadoopConf)
+    }
+
+    if (sparkHttpServer != null) {
+      sparkHttpServer.getClass.getMethod("stop").invoke(sparkHttpServer)
+    }
+    if (sc != null) {
+      sc.stop()
+    }
+    sc = null
+    if (sparkSession != null) {
+      sparkSession.getClass.getMethod("stop").invoke(sparkSession)
+      sparkSession = null
+    }
+    sqlContext = null
+  }
+
+  private def cleanupStagingDirInternal(stagingDirPath: Path, hadoopConf: Configuration): Unit = {
+    try {
+      val fs = stagingDirPath.getFileSystem(hadoopConf)
+      if (fs.delete(stagingDirPath, true)) {
+        LOGGER.info(s"Deleted staging directory $stagingDirPath")
       }
+    } catch {
+      case ioe: IOException =>
+        LOGGER.warn("Failed to cleanup staging dir " + stagingDirPath, ioe)
     }
   }
 
@@ -187,9 +235,7 @@ abstract class BaseSparkScalaInterpreter(val conf: SparkConf,
 
   private def spark1CreateContext(): Unit = {
     this.sc = SparkContext.getOrCreate(conf)
-    if (!isTest) {
-      interpreterOutput.write("Created SparkContext.\n".getBytes())
-    }
+    LOGGER.info("Created SparkContext")
     getUserFiles().foreach(file => sc.addFile(file))
 
     sc.getClass.getMethod("ui").invoke(sc).asInstanceOf[Option[_]] match {
@@ -198,37 +244,33 @@ abstract class BaseSparkScalaInterpreter(val conf: SparkConf,
       case None =>
     }
 
+    initAndSendSparkWebUrl()
+
     val hiveSiteExisted: Boolean =
       Thread.currentThread().getContextClassLoader.getResource("hive-site.xml") != null
-    val hiveEnabled = conf.getBoolean("spark.useHiveContext", false)
+    val hiveEnabled = conf.getBoolean("zeppelin.spark.useHiveContext", false)
     if (hiveEnabled && hiveSiteExisted) {
       sqlContext = Class.forName("org.apache.spark.sql.hive.HiveContext")
         .getConstructor(classOf[SparkContext]).newInstance(sc).asInstanceOf[SQLContext]
-      if (!isTest) {
-        interpreterOutput.write("Created sql context (with Hive support).\n".getBytes())
-      }
+      LOGGER.info("Created sql context (with Hive support)")
     } else {
-      if (hiveEnabled && !hiveSiteExisted && !isTest) {
-        interpreterOutput.write(("spark.useHiveContext is set as true but no hive-site.xml" +
-          " is found in classpath, so zeppelin will fallback to SQLContext.\n").getBytes())
-      }
+      LOGGER.warn("spark.useHiveContext is set as true but no hive-site.xml" +
+          " is found in classpath, so zeppelin will fallback to SQLContext");
       sqlContext = Class.forName("org.apache.spark.sql.SQLContext")
         .getConstructor(classOf[SparkContext]).newInstance(sc).asInstanceOf[SQLContext]
-      if (!isTest) {
-        interpreterOutput.write("Created sql context.\n".getBytes())
-      }
+      LOGGER.info("Created sql context (without Hive support)")
     }
 
     bind("sc", "org.apache.spark.SparkContext", sc, List("""@transient"""))
     bind("sqlContext", sqlContext.getClass.getCanonicalName, sqlContext, List("""@transient"""))
 
-    interpret("import org.apache.spark.SparkContext._")
-    interpret("import sqlContext.implicits._")
-    interpret("import sqlContext.sql")
-    interpret("import org.apache.spark.sql.functions._")
+    scalaInterpret("import org.apache.spark.SparkContext._")
+    scalaInterpret("import sqlContext.implicits._")
+    scalaInterpret("import sqlContext.sql")
+    scalaInterpret("import org.apache.spark.sql.functions._")
     // print empty string otherwise the last statement's output of this method
     // (aka. import org.apache.spark.sql.functions._) will mix with the output of user code
-    interpret("print(\"\")")
+    scalaInterpret("print(\"\")")
   }
 
   private def spark2CreateContext(): Unit = {
@@ -240,7 +282,7 @@ abstract class BaseSparkScalaInterpreter(val conf: SparkConf,
     builder.getClass.getMethod("config", classOf[SparkConf]).invoke(builder, conf)
 
     if (conf.get("spark.sql.catalogImplementation", "in-memory").toLowerCase == "hive"
-      || conf.get("spark.useHiveContext", "false").toLowerCase == "true") {
+      || conf.get("zeppelin.spark.useHiveContext", "false").toLowerCase == "true") {
       val hiveSiteExisted: Boolean =
         Thread.currentThread().getContextClassLoader.getResource("hive-site.xml") != null
       val hiveClassesPresent =
@@ -248,28 +290,20 @@ abstract class BaseSparkScalaInterpreter(val conf: SparkConf,
       if (hiveSiteExisted && hiveClassesPresent) {
         builder.getClass.getMethod("enableHiveSupport").invoke(builder)
         sparkSession = builder.getClass.getMethod("getOrCreate").invoke(builder)
-        if (!isTest) {
-          interpreterOutput.write("Created Spark session (with Hive support).\n".getBytes())
-        }
+        LOGGER.info("Created Spark session (with Hive support)");
       } else {
-        if (!hiveClassesPresent && !isTest) {
-          interpreterOutput.write(
-            "Hive support can not be enabled because spark is not built with hive\n".getBytes)
+        if (!hiveClassesPresent) {
+          LOGGER.warn("Hive support can not be enabled because spark is not built with hive")
         }
-        if (!hiveSiteExisted && !isTest) {
-          interpreterOutput.write(
-            "Hive support can not be enabled because no hive-site.xml found\n".getBytes)
+        if (!hiveSiteExisted) {
+          LOGGER.warn("Hive support can not be enabled because no hive-site.xml found")
         }
         sparkSession = builder.getClass.getMethod("getOrCreate").invoke(builder)
-        if (!isTest) {
-          interpreterOutput.write("Created Spark session.\n".getBytes())
-        }
+        LOGGER.info("Created Spark session (without Hive support)");
       }
     } else {
       sparkSession = builder.getClass.getMethod("getOrCreate").invoke(builder)
-      if (!isTest) {
-        interpreterOutput.write("Created Spark session.\n".getBytes())
-      }
+      LOGGER.info("Created Spark session (without Hive support)");
     }
 
     sc = sparkSession.getClass.getMethod("sparkContext").invoke(sparkSession)
@@ -282,17 +316,64 @@ abstract class BaseSparkScalaInterpreter(val conf: SparkConf,
       case None =>
     }
 
+    initAndSendSparkWebUrl()
+
     bind("spark", sparkSession.getClass.getCanonicalName, sparkSession, List("""@transient"""))
     bind("sc", "org.apache.spark.SparkContext", sc, List("""@transient"""))
     bind("sqlContext", "org.apache.spark.sql.SQLContext", sqlContext, List("""@transient"""))
 
-    interpret("import org.apache.spark.SparkContext._")
-    interpret("import spark.implicits._")
-    interpret("import spark.sql")
-    interpret("import org.apache.spark.sql.functions._")
+    scalaInterpret("import org.apache.spark.SparkContext._")
+    scalaInterpret("import spark.implicits._")
+    scalaInterpret("import spark.sql")
+    scalaInterpret("import org.apache.spark.sql.functions._")
     // print empty string otherwise the last statement's output of this method
     // (aka. import org.apache.spark.sql.functions._) will mix with the output of user code
-    interpret("print(\"\")")
+    scalaInterpret("print(\"\")")
+  }
+
+  private def initAndSendSparkWebUrl(): Unit = {
+    val webUiUrl = properties.getProperty("zeppelin.spark.uiWebUrl");
+    if (!StringUtils.isBlank(webUiUrl)) {
+      this.sparkUrl = webUiUrl.replace("{{applicationId}}", sc.applicationId);
+    } else {
+      useYarnProxyURLIfNeeded()
+    }
+    InterpreterContext.get.getIntpEventClient.sendWebUrlInfo(this.sparkUrl)
+  }
+
+  protected def createZeppelinContext(): Unit = {
+
+    var sparkShims: SparkShims = null
+    if (isSparkSessionPresent()) {
+      sparkShims = SparkShims.getInstance(sc.version, properties, sparkSession)
+    } else {
+      sparkShims = SparkShims.getInstance(sc.version, properties, sc)
+    }
+
+    sparkShims.setupSparkListener(sc.master, sparkUrl, InterpreterContext.get)
+
+    z = new SparkZeppelinContext(sc, sparkShims,
+      interpreterGroup.getInterpreterHookRegistry,
+      properties.getProperty("zeppelin.spark.maxResult", "1000").toInt)
+    bind("z", z.getClass.getCanonicalName, z, List("""@transient"""))
+  }
+
+  private def useYarnProxyURLIfNeeded() {
+    if (properties.getProperty("spark.webui.yarn.useProxy", "false").toBoolean) {
+      if (sparkMaster.startsWith("yarn")) {
+        val appId = sc.applicationId
+        val yarnClient = YarnClient.createYarnClient
+        val yarnConf = new YarnConfiguration()
+        // disable timeline service as we only query yarn app here.
+        // Otherwise we may hit this kind of ERROR:
+        // java.lang.ClassNotFoundException: com.sun.jersey.api.client.config.ClientConfig
+        yarnConf.set("yarn.timeline-service.enabled", "false")
+        yarnClient.init(yarnConf)
+        yarnClient.start()
+        val appReport = yarnClient.getApplicationReport(ConverterUtils.toApplicationId(appId))
+        this.sparkUrl = appReport.getTrackingUrl
+      }
+    }
   }
 
   private def isSparkSessionPresent(): Boolean = {
@@ -392,6 +473,8 @@ abstract class BaseSparkScalaInterpreter(val conf: SparkConf,
         classLoader = classLoader.getParent
       }
     }
+
+    extraJars ++= sparkInterpreterClassLoader.getURLs().map(_.getPath())
     LOGGER.debug("User jar for spark repl: " + extraJars.mkString(","))
     extraJars
   }
