@@ -16,7 +16,6 @@
  *  specific language governing permissions and limitations
  *  under the License.
  */
-
 package org.apache.zeppelin.realm;
 
 import java.io.IOException;
@@ -71,7 +70,6 @@ import org.apache.shiro.util.StringUtils;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
-
 /**
  * Implementation of {@link org.apache.shiro.realm.ldap.JndiLdapRealm} that also returns each user's
  * groups. This implementation is heavily based on org.apache.isis.security.shiro.IsisLdapRealm.
@@ -143,13 +141,12 @@ public class LdapRealm extends JndiLdapRealm {
   private static final String MATCHING_RULE_IN_CHAIN_FORMAT =
       "(&(objectClass=%s)(%s:1.2.840.113556.1.4.1941:=%s))";
 
-  private static Pattern TEMPLATE_PATTERN = Pattern.compile("\\{(\\d+?)\\}");
-  private static String DEFAULT_PRINCIPAL_REGEX = "(.*)";
+  private static final Pattern TEMPLATE_PATTERN = Pattern.compile("\\{(\\d+?)\\}");
+  private static final String DEFAULT_PRINCIPAL_REGEX = "(.*)";
   private static final String MEMBER_SUBSTITUTION_TOKEN = "{0}";
   private static final String HASHING_ALGORITHM = "SHA-1";
   private static final Logger log = LoggerFactory.getLogger(LdapRealm.class);
 
-
   static {
     SUBTREE_SCOPE.setSearchScope(SearchControls.SUBTREE_SCOPE);
     ONELEVEL_SCOPE.setSearchScope(SearchControls.ONELEVEL_SCOPE);
@@ -170,7 +167,6 @@ public class LdapRealm extends JndiLdapRealm {
   private String groupSearchScope = "subtree";
   private boolean groupSearchEnableMatchingRuleInChain;
 
-
   private String groupSearchBase;
 
   private String groupObjectClass = "groupOfNames";
@@ -183,10 +179,9 @@ public class LdapRealm extends JndiLdapRealm {
   private String memberAttributeValuePrefix = "uid=";
   private String memberAttributeValueSuffix = "";
 
-  private final Map<String, String> rolesByGroup = new LinkedHashMap<String, String>();
-  private final List<String> allowedRolesForAuthentication = new ArrayList<String>();
-  private final Map<String, List<String>> permissionsByRole = 
-      new LinkedHashMap<String, List<String>>();
+  private final Map<String, String> rolesByGroup = new LinkedHashMap<>();
+  private final List<String> allowedRolesForAuthentication = new ArrayList<>();
+  private final Map<String, List<String>> permissionsByRole = new LinkedHashMap<>();
 
   private String hadoopSecurityCredentialPath;
   final String keystorePass = "ldapRealm.systemPassword";
@@ -210,7 +205,6 @@ public class LdapRealm extends JndiLdapRealm {
   }
 
   @Override
-
   protected AuthenticationInfo doGetAuthenticationInfo(AuthenticationToken token)
       throws org.apache.shiro.authc.AuthenticationException {
     try {
@@ -222,7 +216,7 @@ public class LdapRealm extends JndiLdapRealm {
 
   protected void onInit() {
     super.onInit();
-    if (!org.apache.commons.lang.StringUtils.isEmpty(this.hadoopSecurityCredentialPath)
+    if (!org.apache.commons.lang3.StringUtils.isEmpty(this.hadoopSecurityCredentialPath)
         && getContextFactory() != null) {
       ((JndiLdapContextFactory) getContextFactory()).setSystemPassword(
           getSystemPassword(this.hadoopSecurityCredentialPath, keystorePass));
@@ -244,7 +238,7 @@ public class LdapRealm extends JndiLdapRealm {
     } catch (IOException e) {
       throw new ShiroException("Error from getting credential entry from keystore", e);
     }
-    if (org.apache.commons.lang.StringUtils.isEmpty(password)) {
+    if (org.apache.commons.lang3.StringUtils.isEmpty(password)) {
       throw new ShiroException("Error getting SystemPassword from the provided keystore:"
           + keystorePass + ", in path:" + hadoopSecurityCredentialPath);
     }
@@ -275,7 +269,7 @@ public class LdapRealm extends JndiLdapRealm {
 
   /**
   * Get groups from LDAP.
-  * 
+  *
   * @param principals
   *            the principals of the Subject whose AuthenticationInfo should
   *            be queried from the LDAP server.
@@ -303,8 +297,7 @@ public class LdapRealm extends JndiLdapRealm {
   }
 
   private boolean hasAllowedAuthenticationRules(PrincipalCollection principals,
-                                                final LdapContextFactory ldapContextFactory)
-      throws NamingException {
+          final LdapContextFactory ldapContextFactory) throws NamingException {
     boolean allowed = allowedRolesForAuthentication.isEmpty();
     if (!allowed) {
       Set<String> roles = getRoles(principals, ldapContextFactory);
@@ -319,9 +312,8 @@ public class LdapRealm extends JndiLdapRealm {
     return allowed;
   }
 
-  private Set<String> getRoles(PrincipalCollection principals, 
-        final LdapContextFactory ldapContextFactory)
-      throws NamingException {
+  private Set<String> getRoles(PrincipalCollection principals,
+          final LdapContextFactory ldapContextFactory) throws NamingException {
     final String username = (String) getAvailablePrincipal(principals);
 
     LdapContext systemLdapCtx = null;
@@ -329,17 +321,17 @@ public class LdapRealm extends JndiLdapRealm {
       systemLdapCtx = ldapContextFactory.getSystemLdapContext();
       return rolesFor(principals, username, systemLdapCtx,
         ldapContextFactory, SecurityUtils.getSubject().getSession());
-    } catch (AuthenticationException ae) {
-      ae.printStackTrace();
+    } catch (Throwable t) {
+      log.warn("Failed to get roles in current context for " + username, t);
       return Collections.emptySet();
     } finally {
       LdapUtils.closeContext(systemLdapCtx);
     }
   }
 
-  protected Set<String> rolesFor(PrincipalCollection principals,
-        String userNameIn, final LdapContext ldapCtx,
-      final LdapContextFactory ldapContextFactory, Session session) throws NamingException {
+  protected Set<String> rolesFor(PrincipalCollection principals, String userNameIn,
+          final LdapContext ldapCtx, final LdapContextFactory ldapContextFactory, Session session)
+          throws NamingException {
     final Set<String> roleNames = new HashSet<>();
     final Set<String> groupNames = new HashSet<>();
     final String userName;
@@ -361,8 +353,8 @@ public class LdapRealm extends JndiLdapRealm {
     byte[] cookie = null;
     try {
       ldapCtx.addToEnvironment(Context.REFERRAL, "ignore");
-        
-      ldapCtx.setRequestControls(new Control[]{new PagedResultsControl(pageSize, 
+      
+      ldapCtx.setRequestControls(new Control[]{new PagedResultsControl(pageSize,
             Control.NONCRITICAL)});
         
       do {
@@ -384,7 +376,7 @@ public class LdapRealm extends JndiLdapRealm {
               final SearchResult group = searchResultEnum.next();
 
               Attribute attribute = group.getAttributes().get(getGroupIdAttribute());
-              String groupName = attribute.get().toString();            
+              String groupName = attribute.get().toString();
               
               String roleName = roleNameFor(groupName);
               if (roleName != null) {
@@ -537,7 +529,7 @@ public class LdapRealm extends JndiLdapRealm {
   }
 
   private Set<String> permsFor(Set<String> roleNames) {
-    Set<String> perms = new LinkedHashSet<String>(); // preserve order
+    Set<String> perms = new LinkedHashSet<>(); // preserve order
     for (String role : roleNames) {
       List<String> permsForRole = permissionsByRole.get(role);
       if (log.isDebugEnabled()) {
@@ -609,7 +601,7 @@ public class LdapRealm extends JndiLdapRealm {
 
   /**
   * Set Member Attribute Template for LDAP.
-  * 
+  *
   * @param template
   *            DN template to be used to query ldap.
   * @throws IllegalArgumentException
@@ -666,7 +658,7 @@ public class LdapRealm extends JndiLdapRealm {
 
   /**
   * Set User Search Attribute Name for LDAP.
-  * 
+  *
   * @param userSearchAttributeName
   *            userAttribute to search ldap.
   */
@@ -711,7 +703,6 @@ public class LdapRealm extends JndiLdapRealm {
   boolean isUserMemberOfDynamicGroup(LdapName userLdapDn, String memberUrl,
       final LdapContextFactory ldapContextFactory) throws NamingException {
     // ldap://host:port/dn?attributes?scope?filter?extensions
-
     if (memberUrl == null) {
       return false;
     }
@@ -727,14 +718,14 @@ public class LdapRealm extends JndiLdapRealm {
     LdapName searchBaseDn = new LdapName(searchBaseString);
 
     // do scope test
-    if (searchScope.equalsIgnoreCase("base")) {
+    if ("base".equalsIgnoreCase(searchScope)) {
       log.debug("DynamicGroup SearchScope base");
       return false;
     }
     if (!userLdapDn.toString().endsWith(searchBaseDn.toString())) {
       return false;
     }
-    if (searchScope.equalsIgnoreCase("one") && (userLdapDn.size() != searchBaseDn.size() - 1)) {
+    if ("one".equalsIgnoreCase(searchScope) && (userLdapDn.size() != searchBaseDn.size() - 1)) {
       log.debug("DynamicGroup SearchScope one");
       return false;
     }
@@ -746,7 +737,7 @@ public class LdapRealm extends JndiLdapRealm {
     NamingEnumeration<SearchResult> searchResultEnum = null;
     try {
       searchResultEnum = systemLdapCtx.search(userLdapDn, searchFilter,
-          searchScope.equalsIgnoreCase("sub") ? SUBTREE_SCOPE : ONELEVEL_SCOPE);
+                                              "sub".equalsIgnoreCase(searchScope) ? SUBTREE_SCOPE : ONELEVEL_SCOPE);
       if (searchResultEnum.hasMore()) {
         return true;
       }
@@ -768,7 +759,7 @@ public class LdapRealm extends JndiLdapRealm {
 
   /**
   * Set Regex for Principal LDAP.
-  * 
+  *
   * @param regex
   *            regex to use to search for principal in shiro.
   */
