@@ -35,6 +35,8 @@ function NotebookCtrl($scope, $route, $routeParams, $location, $rootScope,
   $scope.viewOnly = false;
   $scope.showSetting = false;
   $scope.showRevisionsComparator = false;
+  $scope.collaborativeMode = false;
+  $scope.collaborativeModeUsers = [];
   $scope.looknfeelOption = ['default', 'simple', 'report'];
   $scope.noteFormTitle = null;
   $scope.cronOption = [
@@ -52,6 +54,15 @@ function NotebookCtrl($scope, $route, $routeParams, $location, $rootScope,
     return moment.unix(date).format('MMMM Do YYYY, h:mm a');
   };
 
+  // reorder this array without Head revision, put Head revision in the first
+  $scope.revisionSort = function(array) {
+    let orderArr = array.slice(1).sort((a, b) => {
+      return b.time-a.time;
+    });
+    orderArr.unshift(array[0]);
+    return orderArr;
+  };
+
   $scope.interpreterSettings = [];
   $scope.interpreterBindings = [];
   $scope.isNoteDirty = null;
@@ -101,6 +112,27 @@ function NotebookCtrl($scope, $route, $routeParams, $location, $rootScope,
     connectedOnce = true;
   });
 
+  $scope.addEvent = function(config) {
+    let removeEventByID = function(id) {
+      let events = jQuery._data(config.element, 'events')[config.eventType];
+      if (!events) {
+        return;
+      }
+      for (let i=0; i < events.length; i++) {
+        if (events[i].data && events[i].data.eventID === id) {
+          events.splice(i, 1);
+          i--;
+        }
+      }
+    };
+
+    removeEventByID(config.eventID);
+    angular.element(config.element).bind(config.eventType, {eventID: config.eventID}, config.handler);
+    angular.element(config.onDestroyElement).scope().$on('$destroy', () => {
+      removeEventByID(config.eventID);
+    });
+  };
+
   $scope.getCronOptionNameFromValue = function(value) {
     if (!value) {
       return '';
@@ -214,13 +246,38 @@ function NotebookCtrl($scope, $route, $routeParams, $location, $rootScope,
   };
 
   $scope.isTrash = function(note) {
-    return note ? note.name.split('/')[0] === TRASH_FOLDER_ID : false;
+    return note && note.path ? note.path.split('/')[1] === TRASH_FOLDER_ID : false;
   };
 
   // Export notebook
+  let limit = 0;
+
+  websocketMsgSrv.listConfigurations();
+  $scope.$on('configurationsInfo', function(scope, event) {
+    limit = event.configurations['zeppelin.websocket.max.text.message.size'];
+  });
+
   $scope.exportNote = function() {
-    let jsonContent = JSON.stringify($scope.note);
-    saveAsService.saveAs(jsonContent, $scope.note.name, 'json');
+    let jsonContent = JSON.stringify($scope.note, null, 2);
+    if (jsonContent.length > limit) {
+      BootstrapDialog.confirm({
+        closable: true,
+        title: 'Note size exceeds importable limit (' + limit + ')',
+        message: 'Do you still want to export this note?',
+        callback: function(result) {
+          if (result) {
+            saveAsService.saveAs(jsonContent, $scope.note.name, 'zpln');
+          }
+        },
+      });
+    } else {
+      saveAsService.saveAs(jsonContent, $scope.note.name, 'zpln');
+    }
+  };
+
+  // Export nbformat
+  $scope.exportNbformat = function() {
+    websocketMsgSrv.convertNote($scope.note.id, $scope.note.name);
   };
 
   // Clone note
@@ -284,6 +341,7 @@ function NotebookCtrl($scope, $route, $routeParams, $location, $rootScope,
         $scope.noteRevisions.splice(0, 0, {
           id: 'Head',
           message: 'Head',
+          time: $scope.noteRevisions[0].time,
         });
       }
       if ($routeParams.revisionId) {
@@ -464,12 +522,6 @@ function NotebookCtrl($scope, $route, $routeParams, $location, $rootScope,
     $scope.setConfig();
   };
 
-  /** Set release resource for this note **/
-  $scope.setReleaseResource = function(value) {
-    $scope.note.config.releaseresource = value;
-    $scope.setConfig();
-  };
-
   /** Update note config **/
   $scope.setConfig = function(config) {
     if (config) {
@@ -483,7 +535,7 @@ function NotebookCtrl($scope, $route, $routeParams, $location, $rootScope,
     const trimmedNewName = newName.trim();
     if (trimmedNewName.length > 0 && $scope.note.name !== trimmedNewName) {
       $scope.note.name = trimmedNewName;
-      websocketMsgSrv.renameNote($scope.note.id, $scope.note.name);
+      websocketMsgSrv.renameNote($scope.note.id, $scope.note.name, true);
     }
   };
 
@@ -766,7 +818,7 @@ function NotebookCtrl($scope, $route, $routeParams, $location, $rootScope,
         minimumInputLength: 3,
       };
 
-      $scope.setIamOwner();
+      $scope.setMyPermissions();
       angular.element('#selectOwners').select2(selectJson);
       angular.element('#selectReaders').select2(selectJson);
       angular.element('#selectRunners').select2(selectJson);
@@ -1147,14 +1199,36 @@ function NotebookCtrl($scope, $route, $routeParams, $location, $rootScope,
     }
   };
 
-  $scope.setIamOwner = function() {
-    if ($scope.permissions.owners.length > 0 &&
-      _.indexOf($scope.permissions.owners, $rootScope.ticket.principal) < 0) {
-      $scope.isOwner = false;
-      return false;
+  const arrayIntersection = function(arrayFirst, arraySecond) {
+    return arrayFirst.filter(function(x) {
+      if(arraySecond.indexOf(x) !== -1) {
+        return true;
+      } else {
+        return false;
+      }
+    });
+  };
+
+  $scope.setMyPermissions = function() {
+    let myPermissions;
+    try {
+      myPermissions = JSON.parse($rootScope.ticket.roles);
+    } catch(err) {
+      myPermissions = [];
+    }
+    myPermissions.push($rootScope.ticket.principal);
+
+    $scope.isOwner = !($scope.permissions.owners.length > 0 &&
+       arrayIntersection(myPermissions, $scope.permissions.owners).length === 0);
+
+    $scope.isWriter = !($scope.permissions.writers.length > 0 &&
+       arrayIntersection(myPermissions, $scope.permissions.writers).length === 0);
+
+    if (!$scope.isOwner && !$scope.isWriter) {
+      $scope.viewOnly = true;
+      $scope.note.config.looknfeel = 'report';
+      initializeLookAndFeel();
     }
-    $scope.isOwner = true;
-    return true;
   };
 
   $scope.toggleNotePersonalizedMode = function() {
@@ -1257,6 +1331,16 @@ function NotebookCtrl($scope, $route, $routeParams, $location, $rootScope,
     $scope.saveCursorPosition(paragraph);
   });
 
+  $scope.$on('collaborativeModeStatus', function(event, data) {
+    $scope.collaborativeMode = Boolean(data.status);
+    $scope.collaborativeModeUsers = data.users;
+  });
+
+  $scope.$on('patchReceived', function(event, data) {
+    $scope.collaborativeMode = true;
+  });
+
+
   $scope.$on('runAllBelowAndCurrent', function(event, paragraph, isNeedConfirm) {
     let allParagraphs = $scope.note.paragraphs;
     let toRunParagraphs = [];
@@ -1525,8 +1609,17 @@ function NotebookCtrl($scope, $route, $routeParams, $location, $rootScope,
     document.removeEventListener('keydown', $scope.keyboardShortcut);
   });
 
-  angular.element(window).bind('resize', function() {
-    const actionbarHeight = document.getElementById('actionbar').lastElementChild.clientHeight;
-    angular.element(document.getElementById('content')).css('padding-top', actionbarHeight - 20);
-  });
+  let content = document.getElementById('content');
+  if (content && content.id) {
+    $scope.addEvent({
+      eventID: content.id,
+      eventType: 'resize',
+      element: window,
+      onDestroyElement: content,
+      handler: () => {
+        const actionbarHeight = document.getElementById('actionbar').lastElementChild.clientHeight;
+        angular.element(document.getElementById('content')).css('padding-top', actionbarHeight - 20);
+      },
+    });
+  }
 }
