@@ -16,6 +16,7 @@ import {SpellResult} from '../../spell';
 import {isParagraphRunning, ParagraphStatus} from './paragraph.status';
 
 import moment from 'moment';
+import DiffMatchPatch from 'diff-match-patch';
 
 require('moment-duration-format');
 
@@ -42,6 +43,9 @@ function ParagraphCtrl($scope, $rootScope, $route, $window, $routeParams, $locat
   $scope.paragraph.results.msg = [];
   $scope.originalText = '';
   $scope.editor = null;
+  $scope.cursorPosition = null;
+  $scope.diffMatchPatch = new DiffMatchPatch();
+  $scope.isNoteRunning = false;
 
   // transactional info for spell execution
   $scope.spellTransaction = {
@@ -144,11 +148,20 @@ function ParagraphCtrl($scope, $rootScope, $route, $window, $routeParams, $locat
       $scope.paragraph.config = {};
     }
 
+    $scope.isNoteRunning = !!(note && note.hasOwnProperty('info') &&
+      note.info.hasOwnProperty('isRunning')
+      && note.info.isRunning === true);
+
     noteVarShareService.put($scope.paragraph.id + '_paragraphScope', paragraphScope);
 
     initializeDefault($scope.paragraph.config);
   };
 
+  $scope.$on('noteRunningStatus', function(event, status) {
+    $scope.isNoteRunning = status;
+    $scope.editor.setReadOnly(status);
+  });
+
   const initializeDefault = function(config) {
     let forms = $scope.paragraph.settings.forms;
 
@@ -199,7 +212,7 @@ function ParagraphCtrl($scope, $rootScope, $route, $window, $routeParams, $locat
         $scope.paragraph.results.msg = [];
       }
 
-      let update = ($scope.paragraph.results.msg[data.index]) ? true : false;
+      let update = typeof $scope.paragraph.results.msg[data.index] !== 'undefined';
 
       $scope.paragraph.results.msg[data.index] = {
         data: data.data,
@@ -247,7 +260,7 @@ function ParagraphCtrl($scope, $rootScope, $route, $window, $routeParams, $locat
       console.log('editor isnt loaded yet, returning');
       return;
     }
-    if ($scope.revisionView === true) {
+    if ($scope.revisionView === true || $scope.isNoteRunning === true) {
       $scope.editor.setReadOnly(true);
     } else {
       $scope.editor.setReadOnly(false);
@@ -263,6 +276,9 @@ function ParagraphCtrl($scope, $rootScope, $route, $window, $routeParams, $locat
   };
 
   $scope.cancelParagraph = function(paragraph) {
+    if ($scope.isNoteRunning) {
+      return;
+    }
     console.log('Cancel %o', paragraph.id);
     websocketMsgSrv.cancelParagraphRun(paragraph.id);
   };
@@ -358,9 +374,9 @@ function ParagraphCtrl($scope, $rootScope, $route, $window, $routeParams, $locat
 
     try {
       // remove magic from paragraphText
-      const splited = paragraphText.split(magic);
+      const splited = paragraphText.slice(paragraphText.indexOf(magic) + magic.length);
       // remove leading spaces
-      const textWithoutMagic = splited[1].replace(/^\s+/g, '');
+      const textWithoutMagic = splited.replace(/^\s+/g, '');
 
       if (!propagated) {
         $scope.paragraph.dateStarted = $scope.getFormattedParagraphTime();
@@ -427,6 +443,9 @@ function ParagraphCtrl($scope, $rootScope, $route, $window, $routeParams, $locat
   };
 
   $scope.toggleEnableDisable = function(paragraph) {
+    if ($scope.isNoteRunning) {
+      return;
+    }
     paragraph.config.enabled = !paragraph.config.enabled;
     commitParagraph(paragraph);
   };
@@ -440,6 +459,7 @@ function ParagraphCtrl($scope, $rootScope, $route, $window, $routeParams, $locat
     if (!paragraphText || $scope.isRunning($scope.paragraph)) {
       return;
     }
+
     const magic = SpellResult.extractMagic(paragraphText);
 
     if (heliumService.getSpellByMagic(magic)) {
@@ -469,45 +489,39 @@ function ParagraphCtrl($scope, $rootScope, $route, $window, $routeParams, $locat
   };
 
   $scope.runParagraphFromButton = function() {
+    if ($scope.isNoteRunning) {
+      return;
+    }
     // we come here from the view, so we don't need to call `$digest()`
     $scope.runParagraph($scope.getEditorValue(), false, false);
   };
 
   $scope.runAllToThis = function(paragraph) {
+    if ($scope.isNoteRunning) {
+      return;
+    }
     $scope.$emit('runAllAbove', paragraph, true);
   };
 
   $scope.runAllFromThis = function(paragraph) {
+    if ($scope.isNoteRunning) {
+      return;
+    }
     $scope.$emit('runAllBelowAndCurrent', paragraph, true);
   };
 
-  $scope.runAllToOrFromThis = function(paragraph) {
-    BootstrapDialog.show({
-      message: 'Run paragraphs:',
-      title: '',
-      buttons: [{
-        label: 'Close',
-        action: function(dialog) {
-          dialog.close();
-        },
-      },
-      {
-        label: 'Run all above',
-        cssClass: 'btn-primary',
-        action: function(dialog) {
-          $scope.$emit('runAllAbove', paragraph, false);
-          dialog.close();
-        },
-      },
-      {
-        label: 'Run current and all below',
-        cssClass: 'btn-primary',
-        action: function(dialog) {
-          $scope.$emit('runAllBelowAndCurrent', paragraph, false);
-          dialog.close();
-        },
-      }],
-    });
+  $scope.runAllFromThisFromShortcut = function(paragraph) {
+    if ($scope.isNoteRunning) {
+      return;
+    }
+    $scope.$emit('runAllBelowAndCurrent', paragraph, false);
+  };
+
+  $scope.runAllToThisFromShortcut = function(paragraph) {
+    if ($scope.isNoteRunning) {
+      return;
+    }
+    $scope.$emit('runAllAbove', paragraph, false);
   };
 
   $scope.turnOnAutoRun = function(paragraph) {
@@ -516,18 +530,30 @@ function ParagraphCtrl($scope, $rootScope, $route, $window, $routeParams, $locat
   };
 
   $scope.moveUp = function(paragraph) {
+    if ($scope.isNoteRunning) {
+      return;
+    }
     $scope.$emit('moveParagraphUp', paragraph);
   };
 
   $scope.moveDown = function(paragraph) {
+    if ($scope.isNoteRunning) {
+      return;
+    }
     $scope.$emit('moveParagraphDown', paragraph);
   };
 
   $scope.insertNew = function(position) {
+    if ($scope.isNoteRunning) {
+      return;
+    }
     $scope.$emit('insertParagraph', $scope.paragraph.id, position);
   };
 
   $scope.copyPara = function(position) {
+    if ($scope.isNoteRunning) {
+      return;
+    }
     let editorValue = $scope.getEditorValue();
     if (editorValue) {
       $scope.copyParagraph(editorValue, position);
@@ -560,6 +586,9 @@ function ParagraphCtrl($scope, $rootScope, $route, $window, $routeParams, $locat
   };
 
   $scope.removeParagraph = function(paragraph) {
+    if ($scope.isNoteRunning) {
+      return;
+    }
     if ($scope.note.paragraphs.length === 1) {
       BootstrapDialog.alert({
         closable: true,
@@ -702,9 +731,24 @@ function ParagraphCtrl($scope, $rootScope, $route, $window, $routeParams, $locat
     let dirtyText = session.getValue();
     $scope.dirtyText = dirtyText;
     if ($scope.dirtyText !== $scope.originalText) {
-      $scope.startSaveTimer();
+      if ($scope.collaborativeMode) {
+        $scope.sendPatch();
+      } else {
+        $scope.startSaveTimer();
+      }
     }
     setParagraphMode(session, dirtyText, editor.getCursorPosition());
+    if ($scope.cursorPosition) {
+      editor.moveCursorToPosition($scope.cursorPosition);
+      $scope.cursorPosition = null;
+    }
+  };
+
+  $scope.sendPatch = function() {
+    $scope.originalText = $scope.originalText ? $scope.originalText : '';
+    let patch = $scope.diffMatchPatch.patch_make($scope.originalText, $scope.dirtyText).toString();
+    $scope.originalText = $scope.dirtyText;
+    return websocketMsgSrv.patchParagraph($scope.paragraph.id, $route.current.pathParams.noteId, patch);
   };
 
   $scope.aceLoaded = function(_editor) {
@@ -720,11 +764,11 @@ function ParagraphCtrl($scope, $rootScope, $route, $window, $routeParams, $locat
       $scope.editor.setHighlightActiveLine(false);
       $scope.editor.getSession().setUseWrapMode(true);
       $scope.editor.setTheme('ace/theme/chrome');
-      $scope.editor.setReadOnly($scope.isRunning($scope.paragraph));
+      $scope.editor.setReadOnly($scope.isRunning($scope.paragraph) || $scope.isNoteRunning);
       $scope.editor.setHighlightActiveLine($scope.paragraphFocused);
 
       if ($scope.paragraphFocused) {
-        let prefix = '%' + getInterpreterName($scope.paragraph.text);
+        let prefix = getParagraphMagic($scope.paragraph.text);
         let paragraphText = $scope.paragraph.text ? $scope.paragraph.text.trim() : '';
 
         $scope.editor.focus();
@@ -737,9 +781,10 @@ function ParagraphCtrl($scope, $rootScope, $route, $window, $routeParams, $locat
       }
 
       autoAdjustEditorHeight(_editor);
-      angular.element(window).resize(function() {
-        autoAdjustEditorHeight(_editor);
-      });
+
+      let adjustEditorListener = () => autoAdjustEditorHeight(_editor);
+      angular.element(window).resize(adjustEditorListener);
+      $scope.$on('$destroy', () => angular.element(window).unbind('resize', adjustEditorListener));
 
       if (navigator.appVersion.indexOf('Mac') !== -1) {
         $scope.editor.setKeyboardHandler('ace/keyboard/emacs');
@@ -1059,10 +1104,10 @@ function ParagraphCtrl($scope, $rootScope, $route, $window, $routeParams, $locat
     }
   };
 
-  let getEditorSetting = function(paragraph, interpreterName) {
+  let getEditorSetting = function(paragraph, pragraphText) {
     let deferred = $q.defer();
     if (!$scope.revisionView) {
-      websocketMsgSrv.getEditorSetting(paragraph.id, interpreterName);
+      websocketMsgSrv.getEditorSetting(paragraph.id, pragraphText);
       $timeout(
         $scope.$on('editorSetting', function(event, data) {
           if (paragraph.id === data.paragraphId) {
@@ -1092,10 +1137,10 @@ function ParagraphCtrl($scope, $rootScope, $route, $window, $routeParams, $locat
         !setInterpreterBindings) {
         session.setMode($scope.paragraph.config.editorMode);
       } else {
-        let magic = getInterpreterName(paragraphText);
+        let magic = getParagraphMagic(paragraphText);
         if (editorSetting.magic !== magic) {
           editorSetting.magic = magic;
-          getEditorSetting($scope.paragraph, magic)
+          getEditorSetting($scope.paragraph, paragraphText)
             .then(function(setting) {
               setEditorLanguage(session, setting.editor.language);
               _.merge($scope.paragraph.config.editorSetting, setting.editor);
@@ -1107,8 +1152,9 @@ function ParagraphCtrl($scope, $rootScope, $route, $window, $routeParams, $locat
     setInterpreterBindings = false;
   };
 
-  const getInterpreterName = function(paragraphText) {
-    let intpNameRegexp = /^\s*%(.+?)(\s|\()/g;
+  // return the text that is composed of interpreter name and paragraph properties
+  const getParagraphMagic = function(paragraphText) {
+    let intpNameRegexp = /^\s*(%.+?)(\s)/g;
     let match = intpNameRegexp.exec(paragraphText);
     if (match) {
       return match[1].trim();
@@ -1469,7 +1515,8 @@ function ParagraphCtrl($scope, $rootScope, $route, $window, $routeParams, $locat
     $scope.paragraph.settings = newPara.settings;
     $scope.paragraph.runtimeInfos = newPara.runtimeInfos;
     if ($scope.editor) {
-      $scope.editor.setReadOnly($scope.isRunning(newPara));
+      let isReadOnly = $scope.isRunning(newPara) || $scope.isNoteRunning;
+      $scope.editor.setReadOnly(isReadOnly);
     }
 
     if (!$scope.asIframe) {
@@ -1564,12 +1611,34 @@ function ParagraphCtrl($scope, $rootScope, $route, $window, $routeParams, $locat
     $scope.updateParagraph(oldPara, newPara, updateCallback);
   });
 
+  $scope.$on('patchReceived', function(event, data) {
+    if (data.paragraphId === $scope.paragraph.id) {
+      let patch = data.patch;
+      patch = $scope.diffMatchPatch.patch_fromText(patch);
+      if (!$scope.paragraph.text || $scope.paragraph.text === undefined) {
+        $scope.paragraph.text = '';
+      }
+      $scope.paragraph.text = $scope.diffMatchPatch.patch_apply(patch, $scope.paragraph.text)[0];
+      $scope.originalText = angular.copy($scope.paragraph.text);
+      let newPosition = $scope.editor.getCursorPosition();
+      if (newPosition && newPosition.row && newPosition.column) {
+        $scope.cursorPosition = $scope.editor.getCursorPosition();
+      }
+    }
+  });
+
   $scope.$on('updateProgress', function(event, data) {
     if (data.id === $scope.paragraph.id) {
       $scope.currentProgress = data.progress;
     }
   });
 
+  $scope.$on('updateStatus', function(event, data) {
+    if (data.id === $scope.paragraph.id) {
+      $scope.paragraph.status = data.status;
+    }
+  });
+
   $scope.$on('appendParagraphOutput', function(event, data) {
     if (data.paragraphId === $scope.paragraph.id) {
       if (!$scope.paragraph.results) {
@@ -1610,9 +1679,11 @@ function ParagraphCtrl($scope, $rootScope, $route, $window, $routeParams, $locat
         $timeout(() => $scope.$emit('moveFocusToNextParagraph', paragraphId));
       } else if (!keyEvent.ctrlKey && keyEvent.shiftKey && keyCode === 13) { // Shift + Enter
         $scope.runParagraphFromShortcut($scope.getEditorValue());
-      } else if (keyEvent.ctrlKey && keyEvent.shiftKey && keyCode === 13) { // Ctrl + Shift + Enter
-        $scope.runAllToOrFromThis($scope.paragraph);
-      } else if (keyEvent.ctrlKey && keyEvent.altKey && keyCode === 67) { // Ctrl + Alt + c
+      } else if (keyEvent.ctrlKey && keyEvent.shiftKey && keyCode === 38) { // Ctrl + Shift + UP
+        $scope.runAllToThisFromShortcut($scope.paragraph);
+      } else if (keyEvent.ctrlKey && keyEvent.shiftKey && keyCode === 40) { // Ctrl + Shift + Down
+        $scope.runAllFromThisFromShortcut($scope.paragraph);
+      }else if (keyEvent.ctrlKey && keyEvent.altKey && keyCode === 67) { // Ctrl + Alt + c
         $scope.cancelParagraph($scope.paragraph);
       } else if (keyEvent.ctrlKey && keyEvent.altKey && keyCode === 68) { // Ctrl + Alt + d
         $scope.removeParagraph($scope.paragraph);
