@@ -20,12 +20,17 @@ package org.apache.zeppelin.interpreter;
 import com.google.common.annotations.VisibleForTesting;
 import com.google.common.base.Preconditions;
 import com.google.common.collect.ImmutableMap;
+import com.google.gson.Gson;
+import com.google.gson.GsonBuilder;
 import com.google.gson.JsonArray;
 import com.google.gson.JsonElement;
 import com.google.gson.JsonObject;
 import com.google.gson.annotations.SerializedName;
-import com.google.gson.internal.StringMap;
+import com.google.gson.reflect.TypeToken;
+import com.google.gson.stream.JsonReader;
+import com.google.gson.stream.JsonWriter;
 import org.apache.commons.io.FileUtils;
+import org.apache.commons.lang3.StringUtils;
 import org.apache.zeppelin.conf.ZeppelinConfiguration;
 import org.apache.zeppelin.dep.Dependency;
 import org.apache.zeppelin.dep.DependencyResolver;
@@ -34,34 +39,36 @@ import org.apache.zeppelin.display.AngularObjectRegistryListener;
 import org.apache.zeppelin.helium.ApplicationEventListener;
 import org.apache.zeppelin.interpreter.launcher.InterpreterLaunchContext;
 import org.apache.zeppelin.interpreter.launcher.InterpreterLauncher;
-import org.apache.zeppelin.interpreter.launcher.ShellScriptLauncher;
-import org.apache.zeppelin.interpreter.launcher.SparkInterpreterLauncher;
-import org.apache.zeppelin.interpreter.lifecycle.NullLifecycleManager;
 import org.apache.zeppelin.interpreter.recovery.NullRecoveryStorage;
 import org.apache.zeppelin.interpreter.recovery.RecoveryStorage;
 import org.apache.zeppelin.interpreter.remote.RemoteAngularObjectRegistry;
 import org.apache.zeppelin.interpreter.remote.RemoteInterpreter;
-import org.apache.zeppelin.interpreter.remote.RemoteInterpreterEventPoller;
 import org.apache.zeppelin.interpreter.remote.RemoteInterpreterProcess;
 import org.apache.zeppelin.interpreter.remote.RemoteInterpreterProcessListener;
+import org.apache.zeppelin.plugin.PluginManager;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
 import java.io.File;
 import java.io.FileNotFoundException;
 import java.io.IOException;
+import java.io.StringReader;
+import java.io.StringWriter;
 import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.HashSet;
+import java.util.LinkedHashMap;
 import java.util.LinkedList;
 import java.util.List;
 import java.util.Map;
+import java.util.Map.Entry;
 import java.util.Properties;
 import java.util.Set;
 import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.locks.ReentrantReadWriteLock;
+import java.util.stream.Collectors;
 
-import static org.apache.zeppelin.conf.ZeppelinConfiguration.ConfVars.ZEPPELIN_INTERPRETER_MAX_POOL_SIZE;
+import static org.apache.zeppelin.conf.ZeppelinConfiguration.ConfVars.ZEPPELIN_INTERPRETER_CONNECTION_POOL_SIZE;
 import static org.apache.zeppelin.conf.ZeppelinConfiguration.ConfVars.ZEPPELIN_INTERPRETER_OUTPUT_LIMIT;
 import static org.apache.zeppelin.util.IdHashes.generateId;
 
@@ -77,6 +84,10 @@ public class InterpreterSetting {
       "language", (Object) "text",
       "editOnDblClick", false);
 
+  public static String  PARAGRAPH_CONFIG_RUNONSELECTIONCHANGE = "runOnSelectionChange";
+  public static String  PARAGRAPH_CONFIG_TITLE = "title";
+  public static String  PARAGRAPH_CONFIG_CHECK_EMTPY = "checkEmpty";
+
   private String id;
   private String name;
   // the original interpreter setting template name where it is created from
@@ -95,7 +106,7 @@ public class InterpreterSetting {
    */
   private Object properties = new Properties();
 
-  private Status status;
+  private Status status = Status.READY;
   private String errorReason;
 
   @SerializedName("interpreterGroup")
@@ -121,25 +132,13 @@ public class InterpreterSetting {
   private transient ApplicationEventListener appEventListener;
   private transient DependencyResolver dependencyResolver;
 
-  private transient Map<String, String> infos;
-
-  // Map of the note and paragraphs which has runtime infos generated by this interpreter setting.
-  // This map is used to clear the infos in paragraph when the interpretersetting is restarted
-  private transient Map<String, Set<String>> runtimeInfosToBeCleared;
-
-  private transient ZeppelinConfiguration conf = new ZeppelinConfiguration();
-
-  // TODO(zjffdu) ShellScriptLauncher is the only launcher implemention for now. It could be other
-  // launcher in future when we have other launcher implementation. e.g. third party launcher
-  // service like livy
-  private transient InterpreterLauncher launcher;
-
-  private transient LifecycleManager lifecycleManager;
-  ///////////////////////////////////////////////////////////////////////////////////////////
-
-
+  private transient ZeppelinConfiguration conf = ZeppelinConfiguration.create();
 
   private transient RecoveryStorage recoveryStorage;
+  private transient RemoteInterpreterEventServer interpreterEventServer;
+
+  public static final String CLUSTER_INTERPRETER_LAUNCHER_NAME = "ClusterInterpreterLauncher";
+
   ///////////////////////////////////////////////////////////////////////////////////////////
 
   /**
@@ -218,6 +217,11 @@ public class InterpreterSetting {
       return this;
     }
 
+    public Builder setRemoteInterpreterEventServer(RemoteInterpreterEventServer interpreterEventServer) {
+      interpreterSetting.interpreterEventServer = interpreterEventServer;
+      return this;
+    }
+
     public Builder setRemoteInterpreterProcessListener(RemoteInterpreterProcessListener
                                                        remoteInterpreterProcessListener) {
       interpreterSetting.remoteInterpreterProcessListener = remoteInterpreterProcessListener;
@@ -235,11 +239,6 @@ public class InterpreterSetting {
       return this;
     }
 
-    public Builder setLifecycleManager(LifecycleManager lifecycleManager) {
-      interpreterSetting.lifecycleManager = lifecycleManager;
-      return this;
-    }
-
     public Builder setRecoveryStorage(RecoveryStorage recoveryStorage) {
       interpreterSetting.recoveryStorage = recoveryStorage;
       return this;
@@ -260,11 +259,7 @@ public class InterpreterSetting {
   }
 
   void postProcessing() {
-    this.status = Status.READY;
     this.id = this.name;
-    if (this.lifecycleManager == null) {
-      this.lifecycleManager = new NullLifecycleManager(conf);
-    }
     if (this.recoveryStorage == null) {
       try {
         this.recoveryStorage = new NullRecoveryStorage(conf, interpreterSettingManager);
@@ -285,7 +280,7 @@ public class InterpreterSetting {
     this.name = o.name;
     this.group = o.group;
     this.properties = convertInterpreterProperties(
-        (Map<String, DefaultInterpreterProperty>) o.getProperties());
+        o.getProperties());
     this.interpreterInfos = new ArrayList<>(o.getInterpreterInfos());
     this.option = InterpreterOption.fromInterpreterOption(o.getOption());
     this.dependencies = new ArrayList<>(o.getDependencies());
@@ -294,12 +289,9 @@ public class InterpreterSetting {
     this.conf = o.getConf();
   }
 
-  private void createLauncher() {
-    if (group.equals("spark")) {
-      this.launcher = new SparkInterpreterLauncher(this.conf, this.recoveryStorage);
-    } else {
-      this.launcher = new ShellScriptLauncher(this.conf, this.recoveryStorage);
-    }
+  private InterpreterLauncher createLauncher(Properties properties) throws IOException {
+    return PluginManager.get().loadInterpreterLauncher(
+        getLauncherPlugin(properties), recoveryStorage);
   }
 
   public AngularObjectRegistryListener getAngularObjectRegistryListener() {
@@ -350,28 +342,52 @@ public class InterpreterSetting {
     return this;
   }
 
-  public InterpreterSetting setLifecycleManager(LifecycleManager lifecycleManager) {
-    this.lifecycleManager = lifecycleManager;
-    return this;
-  }
-
   public InterpreterSetting setRecoveryStorage(RecoveryStorage recoveryStorage) {
     this.recoveryStorage = recoveryStorage;
     return this;
   }
 
+  public InterpreterSetting setInterpreterEventServer(
+      RemoteInterpreterEventServer interpreterEventServer) {
+    this.interpreterEventServer = interpreterEventServer;
+    return this;
+  }
+
+  public InterpreterInfo getInterpreterInfo(String name) {
+    for (InterpreterInfo info :interpreterInfos) {
+      if (StringUtils.equals(info.getName(), name)) {
+        return info;
+      }
+    }
+
+    return null;
+  }
+
+  public Map<String, Object> getConfig(String className) {
+    Map<String, Object> configSetting = new HashMap<>();
+    for (InterpreterInfo intpInfo : interpreterInfos) {
+      if (className.equals(intpInfo.getClassName())) {
+         if (intpInfo.getConfig() != null) {
+           configSetting = intpInfo.getConfig();
+         }
+         break;
+      }
+    }
+    return configSetting;
+  }
+
   public RecoveryStorage getRecoveryStorage() {
     return recoveryStorage;
   }
 
-  public LifecycleManager getLifecycleManager() {
-    return lifecycleManager;
-  }
-
   public String getId() {
     return id;
   }
 
+  public void setId(String id) {
+    this.id = id;
+  }
+
   public String getName() {
     return name;
   }
@@ -380,30 +396,44 @@ public class InterpreterSetting {
     return group;
   }
 
-  private String getInterpreterGroupId(String user, String noteId) {
-    String key;
+  private String getInterpreterGroupId(ExecutionContext executionContext) {
+    if (!StringUtils.isBlank(executionContext.getInterpreterGroupId())) {
+      return executionContext.getInterpreterGroupId();
+    }
+
+    if (executionContext.isInIsolatedMode()) {
+      return name + "-isolated-" + executionContext.getNoteId() + "-" +
+              executionContext.getStartTime();
+    }
+
+    List<String> keys = new ArrayList<>();
     if (option.isExistingProcess) {
-      key = Constants.EXISTING_PROCESS;
-    } else if (getOption().isProcess()) {
-      key = (option.perUserIsolated() ? user : "") + ":" + (option.perNoteIsolated() ? noteId : "");
+      keys.add(Constants.EXISTING_PROCESS);
+    } else if (getOption().isIsolated()) {
+      if (option.perUserIsolated()) {
+        keys.add(executionContext.getUser());
+      }
+      if (option.perNoteIsolated()) {
+        keys.add(executionContext.getNoteId());
+      }
     } else {
-      key = SHARED_PROCESS;
+      keys.add(SHARED_PROCESS);
     }
 
     //TODO(zjffdu) we encode interpreter setting id into groupId, this is not a good design
-    return id + ":" + key;
+    return id + "-" + StringUtils.join(keys, "-");
   }
 
-  private String getInterpreterSessionId(String user, String noteId) {
+  private String getInterpreterSessionId(ExecutionContext executionContext) {
     String key;
     if (option.isExistingProcess()) {
       key = Constants.EXISTING_PROCESS;
     } else if (option.perNoteScoped() && option.perUserScoped()) {
-      key = user + ":" + noteId;
+      key = executionContext.getUser() + ":" + executionContext.getNoteId();
     } else if (option.perUserScoped()) {
-      key = user;
+      key = executionContext.getUser();
     } else if (option.perNoteScoped()) {
-      key = noteId;
+      key = executionContext.getNoteId();
     } else {
       key = SHARED_SESSION;
     }
@@ -412,18 +442,22 @@ public class InterpreterSetting {
   }
 
   public ManagedInterpreterGroup getOrCreateInterpreterGroup(String user, String noteId) {
-    String groupId = getInterpreterGroupId(user, noteId);
+    return getOrCreateInterpreterGroup(new ExecutionContextBuilder().setUser(user).setNoteId(noteId).createExecutionContext());
+  }
+
+  public ManagedInterpreterGroup getOrCreateInterpreterGroup(ExecutionContext executionContext) {
+    String groupId = getInterpreterGroupId(executionContext);
     try {
       interpreterGroupWriteLock.lock();
       if (!interpreterGroups.containsKey(groupId)) {
-        LOGGER.info("Create InterpreterGroup with groupId: {} for user: {} and note: {}",
-            groupId, user, noteId);
+        LOGGER.info("Create InterpreterGroup with groupId: {} for {}",
+            groupId, executionContext);
         ManagedInterpreterGroup intpGroup = createInterpreterGroup(groupId);
         interpreterGroups.put(groupId, intpGroup);
       }
       return interpreterGroups.get(groupId);
     } finally {
-      interpreterGroupWriteLock.unlock();;
+      interpreterGroupWriteLock.unlock();
     }
   }
 
@@ -437,12 +471,16 @@ public class InterpreterSetting {
   }
 
   public ManagedInterpreterGroup getInterpreterGroup(String user, String noteId) {
-    String groupId = getInterpreterGroupId(user, noteId);
+    return getInterpreterGroup(new ExecutionContextBuilder().setUser(user).setNoteId(noteId).createExecutionContext());
+  }
+
+  public ManagedInterpreterGroup getInterpreterGroup(ExecutionContext executionContext) {
+    String groupId = getInterpreterGroupId(executionContext);
     try {
       interpreterGroupReadLock.lock();
       return interpreterGroups.get(groupId);
     } finally {
-      interpreterGroupReadLock.unlock();;
+      interpreterGroupReadLock.unlock();
     }
   }
 
@@ -450,10 +488,10 @@ public class InterpreterSetting {
     return interpreterGroups.get(groupId);
   }
 
-  public ArrayList<ManagedInterpreterGroup> getAllInterpreterGroups() {
+  public List<ManagedInterpreterGroup> getAllInterpreterGroups() {
     try {
       interpreterGroupReadLock.lock();
-      return new ArrayList(interpreterGroups.values());
+      return new ArrayList<>(interpreterGroups.values());
     } finally {
       interpreterGroupReadLock.unlock();
     }
@@ -471,37 +509,108 @@ public class InterpreterSetting {
     return DEFAULT_EDITOR;
   }
 
-  void closeInterpreters(String user, String noteId) {
-    ManagedInterpreterGroup interpreterGroup = getInterpreterGroup(user, noteId);
+  public void closeInterpreters(String user, String noteId) {
+    closeInterpreters(new ExecutionContextBuilder().setUser(user).setNoteId(noteId).createExecutionContext());
+  }
+
+  public void closeInterpreters(String interpreterGroupId) {
+    closeInterpreters(new ExecutionContextBuilder().setInterpreterGroupId(interpreterGroupId).createExecutionContext());
+  }
+
+  public void closeInterpreters(ExecutionContext executionContext) {
+    ManagedInterpreterGroup interpreterGroup = getInterpreterGroup(executionContext);
     if (interpreterGroup != null) {
-      String sessionId = getInterpreterSessionId(user, noteId);
+      String sessionId = getInterpreterSessionId(executionContext);
       interpreterGroup.close(sessionId);
     }
   }
 
   public void close() {
-    LOGGER.info("Close InterpreterSetting: " + name);
-    for (ManagedInterpreterGroup intpGroup : interpreterGroups.values()) {
-      intpGroup.close();
-    }
+    LOGGER.info("Close InterpreterSetting: {}", name);
+    List<Thread> closeThreads = interpreterGroups.values().stream()
+            .map(g -> new Thread(g::close, name + "-close"))
+            .peek(t -> t.setUncaughtExceptionHandler((th, e) ->
+                    LOGGER.error("InterpreterSetting close error", e)))
+            .peek(Thread::start)
+            .collect(Collectors.toList());
     interpreterGroups.clear();
-    this.runtimeInfosToBeCleared = null;
-    this.infos = null;
+    for (Thread t : closeThreads) {
+      try {
+        t.join();
+      } catch (InterruptedException e) {
+        LOGGER.error("Can't wait InterpreterSetting close threads", e);
+        Thread.currentThread().interrupt();
+        break;
+      }
+    }
   }
 
   public void setProperties(Object object) {
-    if (object instanceof StringMap) {
-      StringMap<String> map = (StringMap) properties;
-      Properties newProperties = new Properties();
-      for (String key : map.keySet()) {
-        newProperties.put(key, map.get(key));
+    this.properties = object;
+  }
+
+  /**
+   * This is just to fix the issue of ZEPPELIN-4672.
+   * (TODO zjffdu), we should remove these ungly code after we unify the interpreter properties in
+   * interpreter.json & interpreter-setting.json
+   * @param propertiesInTemplate
+   */
+  public void fillPropertyDescription(Object propertiesInTemplate) {
+    if (propertiesInTemplate instanceof Map) {
+      Map<String, DefaultInterpreterProperty> propertiesInTemplate2 =
+              (Map<String, DefaultInterpreterProperty>) propertiesInTemplate;
+      if (this.properties instanceof Map) {
+        Map<String, InterpreterProperty> newInterpreterProperties = (Map)this.properties;
+        for (Map.Entry<String, InterpreterProperty> entry : newInterpreterProperties.entrySet()) {
+          if (propertiesInTemplate2.containsKey(entry.getKey())) {
+            entry.getValue().setDescription(propertiesInTemplate2.get(entry.getKey()).getDescription());
+          }
+        }
+        this.properties = newInterpreterProperties;
       }
-      this.properties = newProperties;
-    } else {
-      this.properties = object;
     }
   }
 
+  /**
+   * This method will sort the properties by the order defined in template.
+   * It is because when interpreter setting is loaded in interpreter-setting.json, it is
+   * still not in correct order.
+   * @param propertiesInTemplate
+   */
+  public void sortPropertiesByTemplate(Object propertiesInTemplate) {
+    if (propertiesInTemplate instanceof LinkedHashMap) {
+      List<String> sortedKeys = new ArrayList(((LinkedHashMap) propertiesInTemplate).keySet());
+      if (this.properties instanceof LinkedHashMap) {
+        LinkedHashMap<String, InterpreterProperty> unSortedProperties = (LinkedHashMap) this.properties;
+        List<String> keys = new ArrayList(unSortedProperties.keySet());
+        keys.sort((o1, o2) -> {
+          int i1 = sortedKeys.indexOf(o1);
+          int i2 = sortedKeys.indexOf(o2);
+          if (i1 != -1 && i2 != -1) {
+            if (i1 < i2) {
+              return -1;
+            } else if (i1 > i2) {
+              return 1;
+            } else {
+              return 0;
+            }
+          } else {
+            if (i1 == -1) {
+              return 1;
+            } else {
+              return -1;
+            }
+          }
+        });
+
+        LinkedHashMap<String, InterpreterProperty> sortedProperties = new LinkedHashMap<>();
+        for (String key : keys) {
+          sortedProperties.put(key, unSortedProperties.get(key));
+        }
+        this.properties = sortedProperties;
+      }
+    }
+  }
 
   public Object getProperties() {
     return properties;
@@ -519,7 +628,8 @@ public class InterpreterSetting {
     Map<String, InterpreterProperty> iProperties = (Map<String, InterpreterProperty>) properties;
     for (Map.Entry<String, InterpreterProperty> entry : iProperties.entrySet()) {
       if (entry.getValue().getValue() != null) {
-        jProperties.setProperty(entry.getKey(), entry.getValue().getValue().toString());
+        jProperties.setProperty(entry.getKey().trim(),
+            entry.getValue().getValue().toString().trim());
       }
     }
 
@@ -528,9 +638,9 @@ public class InterpreterSetting {
           conf.getInt(ZEPPELIN_INTERPRETER_OUTPUT_LIMIT) + "");
     }
 
-    if (!jProperties.containsKey("zeppelin.interpreter.max.poolsize")) {
-      jProperties.setProperty("zeppelin.interpreter.max.poolsize",
-          conf.getInt(ZEPPELIN_INTERPRETER_MAX_POOL_SIZE) + "");
+    if (!jProperties.containsKey(ZEPPELIN_INTERPRETER_CONNECTION_POOL_SIZE.getVarName())) {
+      jProperties.setProperty(ZEPPELIN_INTERPRETER_CONNECTION_POOL_SIZE.getVarName(),
+          conf.getInt(ZEPPELIN_INTERPRETER_CONNECTION_POOL_SIZE) + "");
     }
 
     String interpreterLocalRepoPath = conf.getInterpreterLocalRepoPath();
@@ -555,7 +665,9 @@ public class InterpreterSetting {
 
   public void setDependencies(List<Dependency> dependencies) {
     this.dependencies = dependencies;
-    loadInterpreterDependencies();
+    if (!this.dependencies.isEmpty()) {
+      loadInterpreterDependencies();
+    }
   }
 
   public InterpreterOption getOption() {
@@ -584,7 +696,9 @@ public class InterpreterSetting {
         this.dependencies.add(dependency);
       }
     }
-    loadInterpreterDependencies();
+    if (!dependencies.isEmpty()) {
+      loadInterpreterDependencies();
+    }
   }
 
   void setInterpreterOption(InterpreterOption interpreterOption) {
@@ -617,6 +731,7 @@ public class InterpreterSetting {
   }
 
   public void setStatus(Status status) {
+    LOGGER.info("Set interpreter {} status to {}", name, status);
     this.status = status;
   }
 
@@ -632,14 +747,6 @@ public class InterpreterSetting {
     this.interpreterInfos = interpreterInfos;
   }
 
-  public void setInfos(Map<String, String> infos) {
-    this.infos = infos;
-  }
-
-  public Map<String, String> getInfos() {
-    return infos;
-  }
-
   public InterpreterRunner getInterpreterRunner() {
     return interpreterRunner;
   }
@@ -648,27 +755,54 @@ public class InterpreterSetting {
     this.interpreterRunner = interpreterRunner;
   }
 
-  public void addNoteToPara(String noteId, String paraId) {
-    if (runtimeInfosToBeCleared == null) {
-      runtimeInfosToBeCleared = new HashMap<>();
+  public String getLauncherPlugin(Properties properties) {
+    if (isRunningOnKubernetes()) {
+      return "K8sStandardInterpreterLauncher";
+    } else if (isRunningOnCluster()) {
+      return InterpreterSetting.CLUSTER_INTERPRETER_LAUNCHER_NAME;
+    } else if (isRunningOnDocker()) {
+      return "DockerInterpreterLauncher";
+    } else {
+      String launcher = properties.getProperty("zeppelin.interpreter.launcher");
+      LOGGER.debug("zeppelin.interpreter.launcher: {}", launcher);
+      if (group.equals("spark")) {
+        return "SparkInterpreterLauncher";
+      } else if (group.equals("flink")) {
+        if ("yarn".equals(launcher)) {
+          return "YarnInterpreterLauncher";
+        }
+        return "FlinkInterpreterLauncher";
+      } else {
+        if ("yarn".equals(launcher)) {
+          return "YarnInterpreterLauncher";
+        }
+        return "StandardInterpreterLauncher";
+      }
     }
-    Set<String> paraIdSet = runtimeInfosToBeCleared.get(noteId);
-    if (paraIdSet == null) {
-      paraIdSet = new HashSet<>();
-      runtimeInfosToBeCleared.put(noteId, paraIdSet);
+  }
+
+  private boolean isRunningOnKubernetes() {
+    return conf.getRunMode() == ZeppelinConfiguration.RUN_MODE.K8S;
+  }
+
+
+  private boolean isRunningOnCluster() {
+    return conf.isClusterMode();
+  }
+
+  private boolean isRunningOnDocker() {
+    return conf.getRunMode() == ZeppelinConfiguration.RUN_MODE.DOCKER;
+  }
+
+  public boolean isUserAuthorized(List<String> userAndRoles) {
+    if (!option.permissionIsSet()) {
+      return true;
     }
-    paraIdSet.add(paraId);
+    Set<String> intersection = new HashSet<>(userAndRoles);
+    intersection.retainAll(option.getOwners());
+    return !intersection.isEmpty();
   }
 
-  public Map<String, Set<String>> getNoteIdAndParaMap() {
-    return runtimeInfosToBeCleared;
-  }
-
-  public void clearNoteIdAndParaMap() {
-    runtimeInfosToBeCleared = null;
-  }
-
-
   //////////////////////////// IMPORTANT ////////////////////////////////////////////////
   ///////////////////////////////////////////////////////////////////////////////////////
   ///////////////////////////////////////////////////////////////////////////////////////
@@ -680,7 +814,7 @@ public class InterpreterSetting {
     Properties intpProperties = getJavaProperties();
     for (InterpreterInfo info : interpreterInfos) {
       Interpreter interpreter = new RemoteInterpreter(intpProperties, sessionId,
-          info.getClassName(), user, lifecycleManager);
+          info.getClassName(), user);
       if (info.isDefaultInterpreter()) {
         interpreters.add(0, interpreter);
       } else {
@@ -706,40 +840,46 @@ public class InterpreterSetting {
                                                                  String userName,
                                                                  Properties properties)
       throws IOException {
-    if (launcher == null) {
-      createLauncher();
-    }
+    InterpreterLauncher launcher = createLauncher(properties);
     InterpreterLaunchContext launchContext = new
         InterpreterLaunchContext(properties, option, interpreterRunner, userName,
-        interpreterGroupId, id, group, name);
+        interpreterGroupId, id, group, name, interpreterEventServer.getPort(), interpreterEventServer.getHost());
     RemoteInterpreterProcess process = (RemoteInterpreterProcess) launcher.launch(launchContext);
-    process.setRemoteInterpreterEventPoller(
-        new RemoteInterpreterEventPoller(remoteInterpreterProcessListener, appEventListener));
     recoveryStorage.onInterpreterClientStart(process);
     return process;
   }
 
   List<Interpreter> getOrCreateSession(String user, String noteId) {
-    ManagedInterpreterGroup interpreterGroup = getOrCreateInterpreterGroup(user, noteId);
-    Preconditions.checkNotNull(interpreterGroup, "No InterpreterGroup existed for user {}, " +
-        "noteId {}", user, noteId);
-    String sessionId = getInterpreterSessionId(user, noteId);
-    return interpreterGroup.getOrCreateSession(user, sessionId);
+    return getOrCreateSession(new ExecutionContextBuilder().setUser(user).setNoteId(noteId).createExecutionContext());
+  }
+
+  List<Interpreter> getOrCreateSession(ExecutionContext executionContext) {
+    ManagedInterpreterGroup interpreterGroup = getOrCreateInterpreterGroup(executionContext);
+    Preconditions.checkNotNull(interpreterGroup, "No InterpreterGroup existed for {}", executionContext);
+    String sessionId = getInterpreterSessionId(executionContext);
+    return interpreterGroup.getOrCreateSession(executionContext.getUser(), sessionId);
   }
 
   public Interpreter getDefaultInterpreter(String user, String noteId) {
-    return getOrCreateSession(user, noteId).get(0);
+    return getOrCreateSession(new ExecutionContextBuilder().setUser(user).setNoteId(noteId).createExecutionContext()).get(0);
+  }
+
+  public Interpreter getDefaultInterpreter(ExecutionContext executionContext) {
+    return getOrCreateSession(executionContext).get(0);
   }
 
   public Interpreter getInterpreter(String user, String noteId, String replName) {
-    Preconditions.checkNotNull(noteId, "noteId should be not null");
+    return getInterpreter(new ExecutionContextBuilder().setUser(user).setNoteId(noteId).createExecutionContext(), replName);
+  }
+
+  public Interpreter getInterpreter(ExecutionContext executionContext, String replName) {
     Preconditions.checkNotNull(replName, "replName should be not null");
 
     String className = getInterpreterClassFromInterpreterSetting(replName);
     if (className == null) {
       return null;
     }
-    List<Interpreter> interpreters = getOrCreateSession(user, noteId);
+    List<Interpreter> interpreters = getOrCreateSession(executionContext);
     for (Interpreter interpreter : interpreters) {
       if (className.equals(interpreter.getClassName())) {
         return interpreter;
@@ -805,10 +945,11 @@ public class InterpreterSetting {
     setStatus(Status.DOWNLOADING_DEPENDENCIES);
     setErrorReason(null);
     Thread t = new Thread() {
+      @Override
       public void run() {
         try {
           // dependencies to prevent library conflict
-          File localRepoDir = new File(conf.getInterpreterLocalRepoPath() + "/" + id);
+          File localRepoDir = new File(conf.getInterpreterLocalRepoPath() + '/' + id);
           if (localRepoDir.exists()) {
             try {
               FileUtils.forceDelete(localRepoDir);
@@ -819,10 +960,11 @@ public class InterpreterSetting {
 
           // load dependencies
           List<Dependency> deps = getDependencies();
-          if (deps != null) {
+          if (deps != null && !deps.isEmpty()) {
+            LOGGER.info("Start to download dependencies for interpreter: {}", name);
             for (Dependency d : deps) {
               File destDir = new File(
-                  conf.getRelativeDir(ZeppelinConfiguration.ConfVars.ZEPPELIN_DEP_LOCALREPO));
+                  conf.getAbsoluteDir(ZeppelinConfiguration.ConfVars.ZEPPELIN_DEP_LOCALREPO));
 
               if (d.getExclusions() != null) {
                 dependencyResolver.load(d.getGroupArtifactVersion(), d.getExclusions(),
@@ -832,6 +974,7 @@ public class InterpreterSetting {
                     .load(d.getGroupArtifactVersion(), new File(destDir, id));
               }
             }
+            LOGGER.info("Finish downloading dependencies for interpreter: {}", name);
           }
 
           setStatus(Status.READY);
@@ -844,6 +987,12 @@ public class InterpreterSetting {
           setErrorReason(e.getLocalizedMessage());
           setStatus(Status.ERROR);
         }
+
+        try {
+          interpreterSettingManager.saveToFile();
+        } catch (IOException e) {
+          LOGGER.error("Fail to save interpreter.json", e);
+        }
       }
     };
 
@@ -870,34 +1019,17 @@ public class InterpreterSetting {
 
   // For backward compatibility of interpreter.json format after ZEPPELIN-2403
   static Map<String, InterpreterProperty> convertInterpreterProperties(Object properties) {
-    if (properties != null && properties instanceof StringMap) {
-      Map<String, InterpreterProperty> newProperties = new HashMap<>();
-      StringMap p = (StringMap) properties;
-      for (Object o : p.entrySet()) {
-        Map.Entry entry = (Map.Entry) o;
-        if (!(entry.getValue() instanceof StringMap)) {
-          InterpreterProperty newProperty = new InterpreterProperty(
-              entry.getKey().toString(),
-              entry.getValue(),
-              InterpreterPropertyType.STRING.getValue());
-          newProperties.put(entry.getKey().toString(), newProperty);
-        } else {
-          // already converted
-          return (Map<String, InterpreterProperty>) properties;
-        }
-      }
-      return newProperties;
-
-    } else if (properties instanceof Map) {
+    if (properties instanceof Map) {
       Map<String, Object> dProperties =
           (Map<String, Object>) properties;
-      Map<String, InterpreterProperty> newProperties = new HashMap<>();
-      for (String key : dProperties.keySet()) {
-        Object value = dProperties.get(key);
+      Map<String, InterpreterProperty> newProperties = new LinkedHashMap<>();
+      for (Entry<String, Object> dPropertiesEntry : dProperties.entrySet()) {
+        String key = dPropertiesEntry.getKey();
+        Object value = dPropertiesEntry.getValue();
         if (value instanceof InterpreterProperty) {
           return (Map<String, InterpreterProperty>) properties;
-        } else if (value instanceof StringMap) {
-          StringMap stringMap = (StringMap) value;
+        } else if (value instanceof Map) {
+          Map stringMap = (Map) value;
           InterpreterProperty newProperty = new InterpreterProperty(
               key,
               stringMap.get("value"),
@@ -909,7 +1041,8 @@ public class InterpreterSetting {
           InterpreterProperty property = new InterpreterProperty(
               key,
               dProperty.getValue(),
-              dProperty.getType() != null ? dProperty.getType() : "string"
+              dProperty.getType() != null ? dProperty.getType() : "string",
+              dProperty.getDescription()
               // in case user forget to specify type in interpreter-setting.json
           );
           newProperties.put(key, property);
@@ -927,13 +1060,138 @@ public class InterpreterSetting {
       }
       return newProperties;
     }
-    throw new RuntimeException("Can not convert this type: " + properties.getClass());
+    throw new RuntimeException("Can not convert this type: " + (properties != null ? properties.getClass() : "null"));
   }
 
-  public void waitForReady() throws InterruptedException {
-    while (getStatus().equals(
-        org.apache.zeppelin.interpreter.InterpreterSetting.Status.DOWNLOADING_DEPENDENCIES)) {
-      Thread.sleep(200);
+  public void waitForReady(long timeout) throws InterpreterException {
+    long start = System.currentTimeMillis();
+    while(status != Status.READY) {
+      try {
+        Thread.sleep(1000);
+      } catch (InterruptedException e) {
+        throw new InterpreterException(e);
+      }
+      long now = System.currentTimeMillis();
+      if ((now - start) > timeout) {
+        throw new InterpreterException("Fail to download dependencies in " + timeout / 1000
+                + " seconds");
+      }
     }
   }
+
+  public void waitForReady() throws InterpreterException {
+    waitForReady(Long.MAX_VALUE);
+  }
+
+  public InterpreterInfo getDefaultInterpreterInfo() throws Exception {
+    for (InterpreterInfo interpreterInfo : interpreterInfos) {
+      if (interpreterInfo.isDefaultInterpreter()) {
+        return interpreterInfo;
+      }
+    }
+    if (interpreterInfos.size() == 1) {
+      return interpreterInfos.get(0);
+    }
+    throw new Exception("No default interpreter info found in interpreter setting: " + name);
+  }
+
+  public static String toJson(InterpreterSetting intpSetting) {
+    Gson gson = new GsonBuilder().setPrettyPrinting().create();
+
+    StringWriter stringWriter = new StringWriter();
+    try(JsonWriter jsonWriter = new JsonWriter(stringWriter)){
+      // id
+      jsonWriter.beginObject();
+      jsonWriter.name("id");
+      jsonWriter.value(intpSetting.getId());
+
+      // name
+      jsonWriter.name("name");
+      jsonWriter.value(intpSetting.getName());
+
+      // group
+      jsonWriter.name("group");
+      jsonWriter.value(intpSetting.getGroup());
+
+      // dependencies
+      jsonWriter.name("dependencies");
+      String jsonDep = gson.toJson(intpSetting.getDependencies(), new TypeToken<List<Dependency>>() {
+      }.getType());
+      jsonWriter.value(jsonDep);
+
+      // properties
+      jsonWriter.name("properties");
+      String jsonProps = gson.toJson(intpSetting.getProperties(), new TypeToken<Map<String, InterpreterProperty>>() {
+      }.getType());
+      jsonWriter.value(jsonProps);
+
+      // interpreterOption
+      jsonWriter.name("interpreterOption");
+      String jsonOption = gson.toJson(intpSetting.getOption(), new TypeToken<InterpreterOption>() {
+      }.getType());
+      jsonWriter.value(jsonOption);
+
+      // interpreterGroup
+      jsonWriter.name("interpreterGroup");
+      String jsonIntpInfos = gson.toJson(intpSetting.getInterpreterInfos(), new TypeToken<List<InterpreterInfo>>() {
+      }.getType());
+      jsonWriter.value(jsonIntpInfos);
+
+      jsonWriter.endObject();
+      jsonWriter.flush();
+    } catch (IOException e) {
+      LOGGER.error(e.getMessage(), e);
+    }
+
+    return stringWriter.getBuffer().toString();
+  }
+
+  public static InterpreterSetting fromJson(String json) {
+    Gson gson = new GsonBuilder().setPrettyPrinting().create();
+
+    StringReader stringReader = new StringReader(json);
+    JsonReader jsonReader = new JsonReader(stringReader);
+    InterpreterSetting intpSetting = new InterpreterSetting();
+    try {
+      jsonReader.beginObject();
+      while (jsonReader.hasNext()) {
+        String tag = jsonReader.nextName();
+        if (tag.equals("id")) {
+          String id = jsonReader.nextString();
+          intpSetting.setId(id);
+        } else if (tag.equals("name")) {
+          String name = jsonReader.nextString();
+          intpSetting.setName(name);
+        } else if (tag.equals("group")) {
+          String group = jsonReader.nextString();
+          intpSetting.setGroup(group);
+        } else if (tag.equals("dependencies")) {
+          String strDep = jsonReader.nextString();
+          List<Dependency> dependencies = gson.fromJson(strDep, new TypeToken<List<Dependency>>() {}.getType());
+          intpSetting.setDependencies(dependencies);
+        } else if (tag.equals("properties")) {
+          String strProp = jsonReader.nextString();
+          Map<String, InterpreterProperty> properties = gson.fromJson(strProp,
+              new TypeToken<Map<String, InterpreterProperty>>() {}.getType());
+          intpSetting.setProperties(properties);
+        } else if (tag.equals("interpreterOption")) {
+          String strOption = jsonReader.nextString();
+          InterpreterOption intpOption = gson.fromJson(strOption, new TypeToken<InterpreterOption>() {}.getType());
+          intpSetting.setOption(intpOption);
+        } else if (tag.equals("interpreterGroup")) {
+          String strIntpInfos = jsonReader.nextString();
+          List<InterpreterInfo> intpInfos = gson.fromJson(strIntpInfos, new TypeToken<List<InterpreterInfo>>() {}.getType());
+          intpSetting.setInterpreterInfos(intpInfos);
+        } else {
+          LOGGER.error("Error data type!");
+        }
+      }
+      jsonReader.endObject();
+      jsonReader.close();
+    } catch (IOException e) {
+      LOGGER.error(e.getMessage(), e);
+    }
+
+    return intpSetting;
+  }
 }
