@@ -22,11 +22,10 @@ import org.apache.zeppelin.conf.ZeppelinConfiguration;
 import org.apache.zeppelin.dep.Dependency;
 import org.apache.zeppelin.display.AngularObjectRegistryListener;
 import org.apache.zeppelin.helium.ApplicationEventListener;
-import org.apache.zeppelin.interpreter.lifecycle.NullLifecycleManager;
 import org.apache.zeppelin.interpreter.remote.RemoteInterpreterProcessListener;
 import org.junit.Test;
-import org.sonatype.aether.RepositoryException;
-import org.sonatype.aether.repository.RemoteRepository;
+import org.eclipse.aether.RepositoryException;
+import org.eclipse.aether.repository.RemoteRepository;
 
 import java.io.IOException;
 import java.util.ArrayList;
@@ -35,9 +34,7 @@ import java.util.List;
 import java.util.Map;
 
 import static org.junit.Assert.assertEquals;
-import static org.junit.Assert.assertNotEquals;
 import static org.junit.Assert.assertNotNull;
-import static org.junit.Assert.assertNull;
 import static org.junit.Assert.assertTrue;
 import static org.junit.Assert.fail;
 import static org.mockito.Mockito.mock;
@@ -47,12 +44,11 @@ public class InterpreterSettingManagerTest extends AbstractInterpreterTest {
 
   @Test
   public void testInitInterpreterSettingManager() throws IOException, RepositoryException {
-    assertEquals(5, interpreterSettingManager.get().size());
+    assertEquals(6, interpreterSettingManager.get().size());
     InterpreterSetting interpreterSetting = interpreterSettingManager.getByName("test");
     assertEquals("test", interpreterSetting.getName());
     assertEquals("test", interpreterSetting.getGroup());
-    assertTrue(interpreterSetting.getLifecycleManager() instanceof NullLifecycleManager);
-    assertEquals(3, interpreterSetting.getInterpreterInfos().size());
+    assertEquals(8, interpreterSetting.getInterpreterInfos().size());
     // 3 other builtin properties:
     //   * zeppelin.interpreter.output.limit
     //   * zeppelin.interpreter.localRepo
@@ -74,20 +70,14 @@ public class InterpreterSettingManagerTest extends AbstractInterpreterTest {
     assertEquals(2, repositories.size());
     assertEquals("central", repositories.get(0).getId());
 
-    // verify interpreter binding
-    List<String> interpreterSettingIds = interpreterSettingManager.getInterpreterBinding("2C6793KRV");
-    assertEquals(2, interpreterSettingIds.size());
-    assertEquals("test", interpreterSettingIds.get(0));
-    assertEquals("test2", interpreterSettingIds.get(1));
-
     // Load it again
     InterpreterSettingManager interpreterSettingManager2 = new InterpreterSettingManager(conf,
         mock(AngularObjectRegistryListener.class), mock(RemoteInterpreterProcessListener.class), mock(ApplicationEventListener.class));
-    assertEquals(5, interpreterSettingManager2.get().size());
+    assertEquals(6, interpreterSettingManager2.get().size());
     interpreterSetting = interpreterSettingManager2.getByName("test");
     assertEquals("test", interpreterSetting.getName());
     assertEquals("test", interpreterSetting.getGroup());
-    assertEquals(3, interpreterSetting.getInterpreterInfos().size());
+    assertEquals(8, interpreterSetting.getInterpreterInfos().size());
     assertEquals(6, interpreterSetting.getJavaProperties().size());
     assertEquals("value_1", interpreterSetting.getJavaProperties().getProperty("property_1"));
     assertEquals("new_value_2", interpreterSetting.getJavaProperties().getProperty("property_2"));
@@ -119,7 +109,7 @@ public class InterpreterSettingManagerTest extends AbstractInterpreterTest {
     }
 
     interpreterSettingManager.createNewSetting("test3", "test", new ArrayList<Dependency>(), option, properties);
-    assertEquals(6, interpreterSettingManager.get().size());
+    assertEquals(7, interpreterSettingManager.get().size());
     InterpreterSetting interpreterSetting = interpreterSettingManager.getByName("test3");
     assertEquals("test3", interpreterSetting.getName());
     assertEquals("test", interpreterSetting.getGroup());
@@ -141,11 +131,11 @@ public class InterpreterSettingManagerTest extends AbstractInterpreterTest {
     // load it again, it should be saved in interpreter-setting.json. So we can restore it properly
     InterpreterSettingManager interpreterSettingManager2 = new InterpreterSettingManager(conf,
         mock(AngularObjectRegistryListener.class), mock(RemoteInterpreterProcessListener.class), mock(ApplicationEventListener.class));
-    assertEquals(6, interpreterSettingManager2.get().size());
+    assertEquals(7, interpreterSettingManager2.get().size());
     interpreterSetting = interpreterSettingManager2.getByName("test3");
     assertEquals("test3", interpreterSetting.getName());
     assertEquals("test", interpreterSetting.getGroup());
-    assertEquals(6, interpreterSetting.getJavaProperties().size());
+    assertEquals(4, interpreterSetting.getJavaProperties().size());
     assertEquals("value_4", interpreterSetting.getJavaProperties().getProperty("property_4"));
     assertEquals("scoped", interpreterSetting.getOption().perNote);
     assertEquals("scoped", interpreterSetting.getOption().perUser);
@@ -175,9 +165,6 @@ public class InterpreterSettingManagerTest extends AbstractInterpreterTest {
     assertNotNull(interpreterSetting.getInterpreterSettingManager());
 
     // restart in note page
-    interpreterSettingManager.setInterpreterBinding("user1", "note1", interpreterSettingManager.getSettingIds());
-    interpreterSettingManager.setInterpreterBinding("user2", "note2", interpreterSettingManager.getSettingIds());
-    interpreterSettingManager.setInterpreterBinding("user3", "note3", interpreterSettingManager.getSettingIds());
     // create 3 sessions as it is scoped mode
     interpreterSetting.getOption().setPerUser("scoped");
     interpreterSetting.getDefaultInterpreter("user1", "note1");
@@ -186,105 +173,28 @@ public class InterpreterSettingManagerTest extends AbstractInterpreterTest {
     InterpreterGroup interpreterGroup = interpreterSetting.getInterpreterGroup("user1", "note1");
     assertEquals(3, interpreterGroup.getSessionNum());
     // only close user1's session
-    interpreterSettingManager.restart(interpreterSetting.getId(), "note1", "user1");
+    interpreterSettingManager.restart(interpreterSetting.getId(), "user1", "note1");
     assertEquals(2, interpreterGroup.getSessionNum());
 
     // remove interpreter setting
     interpreterSettingManager.remove(interpreterSetting.getId());
-    assertEquals(5, interpreterSettingManager.get().size());
+    assertEquals(6, interpreterSettingManager.get().size());
 
     // load it again
-    InterpreterSettingManager interpreterSettingManager3 = new InterpreterSettingManager(new ZeppelinConfiguration(),
+    InterpreterSettingManager interpreterSettingManager3 = new InterpreterSettingManager(ZeppelinConfiguration.create(),
         mock(AngularObjectRegistryListener.class), mock(RemoteInterpreterProcessListener.class), mock(ApplicationEventListener.class));
-    assertEquals(5, interpreterSettingManager3.get().size());
+    assertEquals(6, interpreterSettingManager3.get().size());
 
   }
 
   @Test
-  public void testInterpreterBinding() throws IOException {
-    assertNull(interpreterSettingManager.getInterpreterBinding("note1"));
-    interpreterSettingManager.setInterpreterBinding("user1", "note1", interpreterSettingManager.getInterpreterSettingIds());
-    assertEquals(interpreterSettingManager.getInterpreterSettingIds(), interpreterSettingManager.getInterpreterBinding("note1"));
-  }
-
-  @Test
-  public void testUpdateInterpreterBinding_PerNoteShared() throws IOException, InterpreterNotFoundException {
-    InterpreterSetting defaultInterpreterSetting = interpreterSettingManager.get().get(0);
-    defaultInterpreterSetting.getOption().setPerNote("shared");
-
-    interpreterSettingManager.setInterpreterBinding("user1", "note1", interpreterSettingManager.getInterpreterSettingIds());
-    // create interpreter of the first binded interpreter setting
-    interpreterFactory.getInterpreter("user1", "note1", "");
-    assertEquals(1, defaultInterpreterSetting.getAllInterpreterGroups().size());
-
-    // choose the first setting
-    List<String> newSettingIds = new ArrayList<>();
-    newSettingIds.add(interpreterSettingManager.getInterpreterSettingIds().get(1));
-
-    interpreterSettingManager.setInterpreterBinding("user1", "note1", newSettingIds);
-    assertEquals(newSettingIds, interpreterSettingManager.getInterpreterBinding("note1"));
-    // InterpreterGroup will still be alive as it is shared
-    assertEquals(1, defaultInterpreterSetting.getAllInterpreterGroups().size());
-  }
-
-  @Test
-  public void testUpdateInterpreterBinding_PerNoteIsolated() throws IOException, InterpreterNotFoundException {
-    InterpreterSetting defaultInterpreterSetting = interpreterSettingManager.get().get(0);
-    defaultInterpreterSetting.getOption().setPerNote("isolated");
-
-    interpreterSettingManager.setInterpreterBinding("user1", "note1", interpreterSettingManager.getInterpreterSettingIds());
-    // create interpreter of the first binded interpreter setting
-    interpreterFactory.getInterpreter("user1", "note1", "");
-    assertEquals(1, defaultInterpreterSetting.getAllInterpreterGroups().size());
-
-    // choose the first setting
-    List<String> newSettingIds = new ArrayList<>();
-    newSettingIds.add(interpreterSettingManager.getInterpreterSettingIds().get(1));
-
-    interpreterSettingManager.setInterpreterBinding("user1", "note1", newSettingIds);
-    assertEquals(newSettingIds, interpreterSettingManager.getInterpreterBinding("note1"));
-    // InterpreterGroup will be closed as it is only belong to this note
-    assertEquals(0, defaultInterpreterSetting.getAllInterpreterGroups().size());
-
-  }
-
-  @Test
-  public void testUpdateInterpreterBinding_PerNoteScoped() throws IOException, InterpreterNotFoundException {
-    InterpreterSetting defaultInterpreterSetting = interpreterSettingManager.get().get(0);
-    defaultInterpreterSetting.getOption().setPerNote("scoped");
-
-    interpreterSettingManager.setInterpreterBinding("user1", "note1", interpreterSettingManager.getInterpreterSettingIds());
-    interpreterSettingManager.setInterpreterBinding("user1", "note2", interpreterSettingManager.getInterpreterSettingIds());
-    // create 2 interpreter of the first binded interpreter setting for note1 and note2
-    interpreterFactory.getInterpreter("user1", "note1", "");
-    interpreterFactory.getInterpreter("user1", "note2", "");
-    assertEquals(1, defaultInterpreterSetting.getAllInterpreterGroups().size());
-    assertEquals(2, defaultInterpreterSetting.getAllInterpreterGroups().get(0).getSessionNum());
-
-    // choose the first setting
-    List<String> newSettingIds = new ArrayList<>();
-    newSettingIds.add(interpreterSettingManager.getInterpreterSettingIds().get(1));
-
-    interpreterSettingManager.setInterpreterBinding("user1", "note1", newSettingIds);
-    assertEquals(newSettingIds, interpreterSettingManager.getInterpreterBinding("note1"));
-    // InterpreterGroup will be still alive but session belong to note1 will be closed
-    assertEquals(1, defaultInterpreterSetting.getAllInterpreterGroups().size());
-    assertEquals(1, defaultInterpreterSetting.getAllInterpreterGroups().get(0).getSessionNum());
-
-  }
-
-  @Test
-  public void testGetEditor() throws IOException, InterpreterNotFoundException {
-    interpreterSettingManager.setInterpreterBinding("user1", "note1", interpreterSettingManager.getInterpreterSettingIds());
-    Interpreter echoInterpreter = interpreterFactory.getInterpreter("user1", "note1", "test.echo");
+  public void testGetEditor() {
     // get editor setting from interpreter-setting.json
-    Map<String, Object> editor = interpreterSettingManager.getEditorSetting(echoInterpreter, "user1", "note1", "test.echo");
+    Map<String, Object> editor = interpreterSettingManager.getEditorSetting("%test.echo", "note1");
     assertEquals("java", editor.get("language"));
 
-    // when editor setting doesn't exit, return the default editor
-    Interpreter mock1Interpreter = interpreterFactory.getInterpreter("user1", "note1", "mock1");
-    editor = interpreterSettingManager.getEditorSetting(mock1Interpreter,"user1", "note1", "mock1");
-    assertEquals("text", editor.get("language"));
+    editor = interpreterSettingManager.getEditorSetting("%mock1", "note1");
+    assertEquals("python", editor.get("language"));
   }
 
   @Test
@@ -311,7 +221,7 @@ public class InterpreterSettingManagerTest extends AbstractInterpreterTest {
     interpreterSetting.getOrCreateSession("user2", "note2");
     assertEquals(2, interpreterSetting.getAllInterpreterGroups().size());
 
-    interpreterSettingManager.restart(interpreterSetting.getId(), "note1", "user1");
+    interpreterSettingManager.restart(interpreterSetting.getId(), "user1", "note1");
     assertEquals(1, interpreterSetting.getAllInterpreterGroups().size());
   }
 
@@ -325,7 +235,7 @@ public class InterpreterSettingManagerTest extends AbstractInterpreterTest {
     interpreterSetting.getOrCreateSession("user2", "note2");
     assertEquals(2, interpreterSetting.getAllInterpreterGroups().size());
 
-    interpreterSettingManager.restart(interpreterSetting.getId(), "note1", "user1");
+    interpreterSettingManager.restart(interpreterSetting.getId(), "user1", "note1");
     assertEquals(1, interpreterSetting.getAllInterpreterGroups().size());
   }
 
@@ -340,7 +250,7 @@ public class InterpreterSettingManagerTest extends AbstractInterpreterTest {
     assertEquals(1, interpreterSetting.getAllInterpreterGroups().size());
     assertEquals(2, interpreterSetting.getAllInterpreterGroups().get(0).getSessionNum());
 
-    interpreterSettingManager.restart(interpreterSetting.getId(), "note1", "user1");
+    interpreterSettingManager.restart(interpreterSetting.getId(), "user1", "note1");
     assertEquals(1, interpreterSetting.getAllInterpreterGroups().size());
     assertEquals(1, interpreterSetting.getAllInterpreterGroups().get(0).getSessionNum());
   }
@@ -356,8 +266,58 @@ public class InterpreterSettingManagerTest extends AbstractInterpreterTest {
     assertEquals(1, interpreterSetting.getAllInterpreterGroups().size());
     assertEquals(2, interpreterSetting.getAllInterpreterGroups().get(0).getSessionNum());
 
-    interpreterSettingManager.restart(interpreterSetting.getId(), "note1", "user1");
+    interpreterSettingManager.restart(interpreterSetting.getId(), "user1", "note1");
     assertEquals(1, interpreterSetting.getAllInterpreterGroups().size());
     assertEquals(1, interpreterSetting.getAllInterpreterGroups().get(0).getSessionNum());
   }
+
+  @Test
+  public void testInterpreterInclude() throws Exception {
+    try {
+      System.setProperty(ZeppelinConfiguration.ConfVars.ZEPPELIN_INTERPRETER_INCLUDES.getVarName(), "mock1");
+      setUp();
+
+      assertEquals(1, interpreterSettingManager.get().size());
+      assertEquals("mock1", interpreterSettingManager.get().get(0).getGroup());
+    } finally {
+      System.clearProperty(ZeppelinConfiguration.ConfVars.ZEPPELIN_INTERPRETER_INCLUDES.getVarName());
+    }
+  }
+
+  @Test
+  public void testInterpreterExclude() throws Exception {
+    try {
+      System.setProperty(ZeppelinConfiguration.ConfVars.ZEPPELIN_INTERPRETER_EXCLUDES.getVarName(),
+              "test,config_test,mock_resource_pool");
+      setUp();
+
+      assertEquals(2, interpreterSettingManager.get().size());
+      assertNotNull(interpreterSettingManager.getByName("mock1"));
+      assertNotNull(interpreterSettingManager.getByName("mock2"));
+    } finally {
+      System.clearProperty(ZeppelinConfiguration.ConfVars.ZEPPELIN_INTERPRETER_EXCLUDES.getVarName());
+    }
+  }
+
+  @Test
+  public void testInterpreterIncludeExcludeTogether() throws Exception {
+    try {
+      System.setProperty(ZeppelinConfiguration.ConfVars.ZEPPELIN_INTERPRETER_INCLUDES.getVarName(),
+              "test,");
+      System.setProperty(ZeppelinConfiguration.ConfVars.ZEPPELIN_INTERPRETER_EXCLUDES.getVarName(),
+              "config_test,mock_resource_pool");
+
+      try {
+        setUp();
+        fail("Should not able to create InterpreterSettingManager");
+      } catch (Exception e) {
+        e.printStackTrace();
+        assertEquals("zeppelin.interpreter.include and zeppelin.interpreter.exclude can not be specified together, only one can be set.",
+                e.getMessage());
+      }
+    } finally {
+      System.clearProperty(ZeppelinConfiguration.ConfVars.ZEPPELIN_INTERPRETER_INCLUDES.getVarName());
+      System.clearProperty(ZeppelinConfiguration.ConfVars.ZEPPELIN_INTERPRETER_EXCLUDES.getVarName());
+    }
+  }
 }
