@@ -17,69 +17,53 @@
 
 package org.apache.zeppelin.interpreter.remote;
 
+import org.apache.commons.lang3.exception.ExceptionUtils;
 import org.apache.thrift.transport.TTransportException;
+import org.apache.zeppelin.conf.ZeppelinConfiguration;
 import org.apache.zeppelin.display.AngularObjectRegistry;
 import org.apache.zeppelin.display.GUI;
 import org.apache.zeppelin.display.Input;
 import org.apache.zeppelin.display.ui.OptionInput;
-import org.apache.zeppelin.interpreter.*;
+import org.apache.zeppelin.interpreter.AbstractInterpreterTest;
+import org.apache.zeppelin.interpreter.Interpreter;
+import org.apache.zeppelin.interpreter.InterpreterContext;
+import org.apache.zeppelin.interpreter.InterpreterException;
+import org.apache.zeppelin.interpreter.InterpreterOption;
+import org.apache.zeppelin.interpreter.InterpreterResult;
 import org.apache.zeppelin.interpreter.InterpreterResult.Code;
-import org.apache.zeppelin.interpreter.remote.mock.GetAngularObjectSizeInterpreter;
-import org.apache.zeppelin.interpreter.remote.mock.GetEnvPropertyInterpreter;
-import org.apache.zeppelin.user.AuthenticationInfo;
-import org.junit.After;
+import org.apache.zeppelin.interpreter.InterpreterSetting;
 import org.junit.Before;
 import org.junit.Test;
 
 import java.io.IOException;
 import java.util.ArrayList;
-import java.util.HashMap;
 import java.util.LinkedHashMap;
 import java.util.List;
 import java.util.Map;
 
-import static org.junit.Assert.*;
+import static org.junit.Assert.assertArrayEquals;
+import static org.junit.Assert.assertEquals;
+import static org.junit.Assert.assertFalse;
+import static org.junit.Assert.assertNotEquals;
+import static org.junit.Assert.assertNotNull;
+import static org.junit.Assert.assertNull;
+import static org.junit.Assert.assertTrue;
+import static org.junit.Assert.fail;
 
-public class RemoteInterpreterTest {
-
-
-  private static final String INTERPRETER_SCRIPT =
-      System.getProperty("os.name").startsWith("Windows") ?
-          "../bin/interpreter.cmd" :
-          "../bin/interpreter.sh";
+public class RemoteInterpreterTest extends AbstractInterpreterTest {
 
   private InterpreterSetting interpreterSetting;
 
+  @Override
   @Before
   public void setUp() throws Exception {
-    InterpreterOption interpreterOption = new InterpreterOption();
-
-    InterpreterInfo interpreterInfo1 = new InterpreterInfo(EchoInterpreter.class.getName(), "echo", true, new HashMap<String, Object>());
-    InterpreterInfo interpreterInfo2 = new InterpreterInfo(DoubleEchoInterpreter.class.getName(), "double_echo", false, new HashMap<String, Object>());
-    InterpreterInfo interpreterInfo3 = new InterpreterInfo(SleepInterpreter.class.getName(), "sleep", false, new HashMap<String, Object>());
-    InterpreterInfo interpreterInfo4 = new InterpreterInfo(GetEnvPropertyInterpreter.class.getName(), "get", false, new HashMap<String, Object>());
-    InterpreterInfo interpreterInfo5 = new InterpreterInfo(GetAngularObjectSizeInterpreter.class.getName(), "angular_obj",false, new HashMap<String, Object>());
-    List<InterpreterInfo> interpreterInfos = new ArrayList<>();
-    interpreterInfos.add(interpreterInfo1);
-    interpreterInfos.add(interpreterInfo2);
-    interpreterInfos.add(interpreterInfo3);
-    interpreterInfos.add(interpreterInfo4);
-    interpreterInfos.add(interpreterInfo5);
-    InterpreterRunner runner = new InterpreterRunner(INTERPRETER_SCRIPT, INTERPRETER_SCRIPT);
-    interpreterSetting = new InterpreterSetting.Builder()
-        .setId("test")
-        .setName("test")
-        .setGroup("test")
-        .setInterpreterInfos(interpreterInfos)
-        .setOption(interpreterOption)
-        .setRunner(runner)
-        .setInterpreterDir("../interpeters/test")
-        .create();
+    super.setUp();
+    interpreterSetting = interpreterSettingManager.getInterpreterSettingByName("test");
   }
 
-  @After
+  @Override
   public void tearDown() throws Exception {
-    interpreterSetting.close();
+    super.tearDown();
   }
 
   @Test
@@ -95,9 +79,7 @@ public class RemoteInterpreterTest {
 
     assertEquals(remoteInterpreter1.getScheduler(), remoteInterpreter2.getScheduler());
 
-    InterpreterContext context1 = new InterpreterContext("noteId", "paragraphId", "repl",
-        "title", "text", AuthenticationInfo.ANONYMOUS, new HashMap<String, Object>(), new GUI(), new GUI(),
-        null, null, new ArrayList<InterpreterContextRunner>(), null);
+    InterpreterContext context1 = createDummyInterpreterContext();
     assertEquals("hello", remoteInterpreter1.interpret("hello", context1).message().get(0).getData());
     assertEquals(Interpreter.FormType.NATIVE, interpreter1.getFormType());
     assertEquals(0, remoteInterpreter1.getProgress(context1));
@@ -112,19 +94,10 @@ public class RemoteInterpreterTest {
     // RemoteInterpreterProcess leakage.
     remoteInterpreter1.getInterpreterGroup().close(remoteInterpreter1.getSessionId());
     assertNull(remoteInterpreter1.getInterpreterGroup().getRemoteInterpreterProcess());
-    try {
-      assertEquals("hello", remoteInterpreter1.interpret("hello", context1).message().get(0).getData());
-      fail("Should not be able to call interpret after interpreter is closed");
-    } catch (Exception e) {
-      e.printStackTrace();
-    }
 
-    try {
-      assertEquals("hello", remoteInterpreter2.interpret("hello", context1).message().get(0).getData());
-      fail("Should not be able to call getProgress after RemoterInterpreterProcess is stoped");
-    } catch (Exception e) {
-      e.printStackTrace();
-    }
+    InterpreterResult result = remoteInterpreter1.interpret("hello", context1);
+    assertEquals(Code.ERROR, result.code());
+    assertEquals("Interpreter process is not running\n", result.message().get(0).getData());
   }
 
   @Test
@@ -140,9 +113,7 @@ public class RemoteInterpreterTest {
 
     assertNotEquals(interpreter1.getScheduler(), interpreter2.getScheduler());
 
-    InterpreterContext context1 = new InterpreterContext("noteId", "paragraphId", "repl",
-        "title", "text", AuthenticationInfo.ANONYMOUS, new HashMap<String, Object>(), new GUI(), new GUI(),
-        null, null, new ArrayList<InterpreterContextRunner>(), null);
+    InterpreterContext context1 = createDummyInterpreterContext();
     assertEquals("hello", remoteInterpreter1.interpret("hello", context1).message().get(0).getData());
     assertEquals("hello", remoteInterpreter2.interpret("hello", context1).message().get(0).getData());
     assertEquals(Interpreter.FormType.NATIVE, interpreter1.getFormType());
@@ -166,12 +137,10 @@ public class RemoteInterpreterTest {
     assertTrue(remoteInterpreter2.getInterpreterGroup().getRemoteInterpreterProcess().isRunning());
     assertEquals("hello", remoteInterpreter2.interpret("hello", context1).message().get(0).getData());
     remoteInterpreter2.getInterpreterGroup().close(remoteInterpreter2.getSessionId());
-    try {
-      assertEquals("hello", remoteInterpreter2.interpret("hello", context1));
-      fail("Should not be able to call interpret after interpreter is closed");
-    } catch (Exception e) {
-      e.printStackTrace();
-    }
+
+    InterpreterResult result = remoteInterpreter2.interpret("hello", context1);
+    assertEquals(Code.ERROR, result.code());
+    assertEquals("Interpreter process is not running\n", result.message().get(0).getData());
     assertNull(remoteInterpreter2.getInterpreterGroup().getRemoteInterpreterProcess());
   }
 
@@ -188,9 +157,7 @@ public class RemoteInterpreterTest {
 
     assertNotEquals(interpreter1.getScheduler(), interpreter2.getScheduler());
 
-    InterpreterContext context1 = new InterpreterContext("noteId", "paragraphId", "repl",
-        "title", "text", AuthenticationInfo.ANONYMOUS, new HashMap<String, Object>(), new GUI(), new GUI(),
-        null, null, new ArrayList<InterpreterContextRunner>(), null);
+    InterpreterContext context1 = createDummyInterpreterContext();
     assertEquals("hello", remoteInterpreter1.interpret("hello", context1).message().get(0).getData());
     assertEquals("hello", remoteInterpreter2.interpret("hello", context1).message().get(0).getData());
     assertEquals(Interpreter.FormType.NATIVE, interpreter1.getFormType());
@@ -205,21 +172,18 @@ public class RemoteInterpreterTest {
     remoteInterpreter1.getInterpreterGroup().close(remoteInterpreter1.getSessionId());
     assertNull(remoteInterpreter1.getInterpreterGroup().getRemoteInterpreterProcess());
     assertTrue(remoteInterpreter2.getInterpreterGroup().getRemoteInterpreterProcess().isRunning());
-    try {
-      remoteInterpreter1.interpret("hello", context1);
-      fail("Should not be able to call getProgress after interpreter is closed");
-    } catch (Exception e) {
-      e.printStackTrace();
-    }
+
+    InterpreterResult result = remoteInterpreter1.interpret("hello", context1);
+    assertEquals(Code.ERROR, result.code());
+    assertEquals("Interpreter process is not running\n", result.message().get(0).getData());
 
     assertEquals("hello", remoteInterpreter2.interpret("hello", context1).message().get(0).getData());
     remoteInterpreter2.getInterpreterGroup().close(remoteInterpreter2.getSessionId());
-    try {
-      assertEquals("hello", remoteInterpreter2.interpret("hello", context1).message().get(0).getData());
-      fail("Should not be able to call interpret after interpreter is closed");
-    } catch (Exception e) {
-      e.printStackTrace();
-    }
+
+    result = remoteInterpreter2.interpret("hello", context1);
+    assertEquals(Code.ERROR, result.code());
+    assertEquals("Interpreter process is not running\n", result.message().get(0).getData());
+
     assertNull(remoteInterpreter2.getInterpreterGroup().getRemoteInterpreterProcess());
 
   }
@@ -229,9 +193,7 @@ public class RemoteInterpreterTest {
     interpreterSetting.getOption().setPerUser(InterpreterOption.SHARED);
     interpreterSetting.setProperty("zeppelin.SleepInterpreter.precode", "fail test");
     Interpreter interpreter1 = interpreterSetting.getInterpreter("user1", "note1", "sleep");
-    InterpreterContext context1 = new InterpreterContext("noteId", "paragraphId", "repl",
-        "title", "text", AuthenticationInfo.ANONYMOUS, new HashMap<String, Object>(), new GUI(), new GUI(),
-        null, null, new ArrayList<InterpreterContextRunner>(), null);
+    InterpreterContext context1 = createDummyInterpreterContext();;
     assertEquals(Code.ERROR, interpreter1.interpret("10", context1).code());
   }
 
@@ -240,9 +202,7 @@ public class RemoteInterpreterTest {
     interpreterSetting.getOption().setPerUser(InterpreterOption.SHARED);
     interpreterSetting.setProperty("zeppelin.SleepInterpreter.precode", "1");
     Interpreter interpreter1 = interpreterSetting.getInterpreter("user1", "note1", "sleep");
-    InterpreterContext context1 = new InterpreterContext("noteId", "paragraphId", "repl",
-        "title", "text", AuthenticationInfo.ANONYMOUS, new HashMap<String, Object>(), new GUI(), new GUI(),
-        null, null, new ArrayList<InterpreterContextRunner>(), null);
+    InterpreterContext context1 = createDummyInterpreterContext();
     assertEquals(Code.SUCCESS, interpreter1.interpret("10", context1).code());
   }
 
@@ -255,9 +215,7 @@ public class RemoteInterpreterTest {
     assertTrue(interpreter1 instanceof RemoteInterpreter);
     RemoteInterpreter remoteInterpreter1 = (RemoteInterpreter) interpreter1;
 
-    InterpreterContext context1 = new InterpreterContext("noteId", "paragraphId", "repl",
-        "title", "text", AuthenticationInfo.ANONYMOUS, new HashMap<String, Object>(), new GUI(), new GUI(),
-        null, null, new ArrayList<InterpreterContextRunner>(), null);
+    InterpreterContext context1 = createDummyInterpreterContext();;
     assertEquals(Code.ERROR, remoteInterpreter1.interpret("hello", context1).code());
   }
 
@@ -267,9 +225,7 @@ public class RemoteInterpreterTest {
     // by default SleepInterpreter would use FIFOScheduler
 
     final Interpreter interpreter1 = interpreterSetting.getInterpreter("user1", "note1", "sleep");
-    final InterpreterContext context1 = new InterpreterContext("noteId", "paragraphId", "repl",
-        "title", "text", AuthenticationInfo.ANONYMOUS, new HashMap<String, Object>(), new GUI(), new GUI(),
-        null, null, new ArrayList<InterpreterContextRunner>(), null);
+    final InterpreterContext context1 = createDummyInterpreterContext();
     // run this dummy interpret method first to launch the RemoteInterpreterProcess to avoid the
     // time overhead of launching the process.
     interpreter1.interpret("1", context1);
@@ -310,9 +266,7 @@ public class RemoteInterpreterTest {
     interpreterSetting.setProperty("zeppelin.SleepInterpreter.parallel", "true");
 
     final Interpreter interpreter1 = interpreterSetting.getInterpreter("user1", "note1", "sleep");
-    final InterpreterContext context1 = new InterpreterContext("noteId", "paragraphId", "repl",
-        "title", "text", AuthenticationInfo.ANONYMOUS, new HashMap<String, Object>(), new GUI(), new GUI(),
-        null, null, new ArrayList<InterpreterContextRunner>(), null);
+    final InterpreterContext context1 = createDummyInterpreterContext();
 
     // run this dummy interpret method first to launch the RemoteInterpreterProcess to avoid the
     // time overhead of launching the process.
@@ -383,10 +337,7 @@ public class RemoteInterpreterTest {
     Interpreter interpreter = interpreterSetting.getInterpreter("user1", "note1", "angular_obj");
     interpreter.getInterpreterGroup().setAngularObjectRegistry(registry);
 
-    final InterpreterContext context = new InterpreterContext("noteId", "paragraphId", "repl",
-        "title", "text", AuthenticationInfo.ANONYMOUS, new HashMap<String, Object>(), new GUI(), new GUI(),
-        null, null, new ArrayList<InterpreterContextRunner>(), null);
-
+    final InterpreterContext context = createDummyInterpreterContext();
     InterpreterResult result = interpreter.interpret("dummy", context);
     assertEquals(Code.SUCCESS, result.code());
     assertEquals("2", result.message().get(0).getData());
@@ -410,15 +361,13 @@ public class RemoteInterpreterTest {
     interpreterSetting.setProperty("property_1", "value_1");
 
     final Interpreter interpreter1 = interpreterSetting.getInterpreter("user1", "note1", "get");
-    final InterpreterContext context1 = new InterpreterContext("noteId", "paragraphId", "repl",
-        "title", "text", AuthenticationInfo.ANONYMOUS, new HashMap<String, Object>(), new GUI(), new GUI(),
-        null, null, new ArrayList<InterpreterContextRunner>(), null);
+    final InterpreterContext context1 = createDummyInterpreterContext();
 
     assertEquals("VALUE_1", interpreter1.interpret("getEnv ENV_1", context1).message().get(0).getData());
     assertEquals("null", interpreter1.interpret("getEnv ENV_2", context1).message().get(0).getData());
 
     assertEquals("value_1", interpreter1.interpret("getProperty property_1", context1).message().get(0).getData());
-    assertEquals("null", interpreter1.interpret("getProperty property_2", context1).message().get(0).getData());
+    assertEquals("null", interpreter1.interpret("getProperty not_existed_property", context1).message().get(0).getData());
   }
 
   @Test
@@ -428,20 +377,78 @@ public class RemoteInterpreterTest {
         new OptionInput.ParamOption("value1", "param1"),
         new OptionInput.ParamOption("value2", "param2")
     };
-    List<Object> defaultValues = new ArrayList();
+    List<Object> defaultValues = new ArrayList<>();
     defaultValues.add("default1");
     defaultValues.add("default2");
-    gui.checkbox("checkbox_id", defaultValues, paramOptions);
-    gui.select("select_id", "default", paramOptions);
+    gui.checkbox("checkbox_id", paramOptions, defaultValues);
+    gui.select("select_id", paramOptions, "default");
     gui.textbox("textbox_id");
     Map<String, Input> expected = new LinkedHashMap<>(gui.getForms());
     Interpreter interpreter = interpreterSetting.getDefaultInterpreter("user1", "note1");
-    InterpreterContext context = new InterpreterContext("noteId", "paragraphId", "repl", null,
-        null, AuthenticationInfo.ANONYMOUS, new HashMap<String, Object>(), gui, new GUI(),
-        null, null, new ArrayList<InterpreterContextRunner>(), null);
+    InterpreterContext context = createDummyInterpreterContext();
 
     interpreter.interpret("text", context);
     assertArrayEquals(expected.values().toArray(), gui.getForms().values().toArray());
   }
 
+  @Test
+  public void testFailToLaunchInterpreterProcess_InvalidRunner() {
+    try {
+      System.setProperty(ZeppelinConfiguration.ConfVars.ZEPPELIN_INTERPRETER_REMOTE_RUNNER.getVarName(), "invalid_runner");
+      final Interpreter interpreter1 = interpreterSetting.getInterpreter("user1", "note1", "sleep");
+      final InterpreterContext context1 = createDummyInterpreterContext();
+      // run this dummy interpret method first to launch the RemoteInterpreterProcess to avoid the
+      // time overhead of launching the process.
+      try {
+        interpreter1.interpret("1", context1);
+        fail("Should not be able to launch interpreter process");
+      } catch (InterpreterException e) {
+        assertTrue(ExceptionUtils.getStackTrace(e).contains("java.io.IOException"));
+      }
+    } finally {
+      System.clearProperty(ZeppelinConfiguration.ConfVars.ZEPPELIN_INTERPRETER_REMOTE_RUNNER.getVarName());
+    }
+  }
+
+  @Test
+  public void testFailToLaunchInterpreterProcess_ErrorInRunner() {
+    try {
+      System.setProperty(ZeppelinConfiguration.ConfVars.ZEPPELIN_INTERPRETER_REMOTE_RUNNER.getVarName(),
+               zeppelinHome.getAbsolutePath() + "/zeppelin-zengine/src/test/resources/bin/interpreter_invalid.sh");
+      final Interpreter interpreter1 = interpreterSetting.getInterpreter("user1", "note1", "sleep");
+      final InterpreterContext context1 = createDummyInterpreterContext();
+      // run this dummy interpret method first to launch the RemoteInterpreterProcess to avoid the
+      // time overhead of launching the process.
+      try {
+        interpreter1.interpret("1", context1);
+        fail("Should not be able to launch interpreter process");
+      } catch (InterpreterException e) {
+        assertTrue(ExceptionUtils.getStackTrace(e).contains("invalid_command:"));
+      }
+    } finally {
+      System.clearProperty(ZeppelinConfiguration.ConfVars.ZEPPELIN_INTERPRETER_REMOTE_RUNNER.getVarName());
+    }
+  }
+
+  @Test
+  public void testFailToLaunchInterpreterProcess_Timeout() {
+    try {
+      System.setProperty(ZeppelinConfiguration.ConfVars.ZEPPELIN_INTERPRETER_REMOTE_RUNNER.getVarName(),
+              zeppelinHome.getAbsolutePath() + "/zeppelin-zengine/src/test/resources/bin/interpreter_timeout.sh");
+      System.setProperty(ZeppelinConfiguration.ConfVars.ZEPPELIN_INTERPRETER_CONNECT_TIMEOUT.getVarName(), "10000");
+      final Interpreter interpreter1 = interpreterSetting.getInterpreter("user1", "note1", "sleep");
+      final InterpreterContext context1 = createDummyInterpreterContext();
+      // run this dummy interpret method first to launch the RemoteInterpreterProcess to avoid the
+      // time overhead of launching the process.
+      try {
+        interpreter1.interpret("1", context1);
+        fail("Should not be able to launch interpreter process");
+      } catch (InterpreterException e) {
+        assertTrue(ExceptionUtils.getStackTrace(e).contains("Interpreter Process creation is time out"));
+      }
+    } finally {
+      System.clearProperty(ZeppelinConfiguration.ConfVars.ZEPPELIN_INTERPRETER_REMOTE_RUNNER.getVarName());
+      System.clearProperty(ZeppelinConfiguration.ConfVars.ZEPPELIN_INTERPRETER_CONNECT_TIMEOUT.getVarName());
+    }
+  }
 }
